from typing import Annotated, List
from uuid import UUID

from rapid_api_client import PydanticBody

from remnawave.models import (
    BulkDeleteHostsResponseDto,
    BulkDisableHostsResponseDto,
    BulkEnableHostsResponseDto,
    SetInboundToManyHostsRequestDto,
    SetInboundToManyHostsResponseDto,
    SetPortToManyHostsResponseDto,
    SetPortToManyHostsRequestDto
)
from remnawave.rapid import AttributeBody, BaseController, post


class HostsBulkActionsController(BaseController):
    @post("/hosts/bulk/delete", response_class=BulkDeleteHostsResponseDto)
    async def delete_hosts(
        self,
        uuids: Annotated[List[UUID], AttributeBody()],
    ) -> BulkDeleteHostsResponseDto:
        """Delete many hosts"""
        ...

    @post("/hosts/bulk/disable", response_class=BulkDisableHostsResponseDto)
    async def disable_hosts(
        self,
        uuids: Annotated[List[UUID], AttributeBody()],
    ) -> BulkDisableHostsResponseDto:
        """Disable many hosts"""
        ...

    @post("/hosts/bulk/enable", response_class=BulkEnableHostsResponseDto)
    async def enable_hosts(
        self,
        uuids: Annotated[List[UUID], AttributeBody()],
    ) -> BulkEnableHostsResponseDto:
        """Enable many hosts"""
        ...

    @post(
        "/hosts/bulk/set-inbound",
        response_class=SetInboundToManyHostsResponseDto,
    )
    async def set_inbound_to_hosts(
        self,
        body: Annotated[SetInboundToManyHostsRequestDto, PydanticBody()],
    ) -> SetInboundToManyHostsResponseDto:
        """Set inbound to many hosts"""
        ...

    @post("/hosts/bulk/set-port", response_class=SetPortToManyHostsResponseDto)
    async def set_port_to_hosts(
        self,
        body: Annotated[SetPortToManyHostsRequestDto, PydanticBody()],
    ) -> SetPortToManyHostsResponseDto:
        """Set port to many hosts"""
        ...



from typing import Any, List, Optional
from uuid import UUID

from pydantic import BaseModel, Field, RootModel


class InboundResponseDto(BaseModel):
    uuid: UUID
    profile_uuid: UUID = Field(alias="profileUuid")
    tag: str
    type: str
    network: Optional[str] = None
    security: Optional[str] = None
    port: Optional[float] = None
    raw_inbound: Optional[Any] = Field(None, alias="rawInbound")
    active_squads: Optional[list[UUID]] = Field(None, alias="activeSquads")


class AllInboundsData(BaseModel):
    total: int
    inbounds: List[InboundResponseDto]


class GetAllInboundsResponseDto(AllInboundsData):
    pass


class InboundsByProfileData(BaseModel):
    total: int
    inbounds: List[InboundResponseDto]


class GetInboundsByProfileUuidResponseDto(InboundsByProfileData):
    pass


# Legacy models for backward compatibility
class GetInboundsResponseDto(RootModel[List[InboundResponseDto]]):
    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]
    
    def __bool__(self):
        """Return True if list is not empty"""
        return bool(self.root)
    
    def __len__(self):
        """Return length of list"""
        return len(self.root)


class FullInboundStatistic(BaseModel):
    enabled: float
    disabled: float


class FullInboundResponseDto(BaseModel):
    uuid: UUID
    tag: str
    type: str
    port: float
    network: Optional[str] = None
    security: Optional[str] = None
    raw_from_config: Any = Field(alias="rawFromConfig")
    users: FullInboundStatistic
    nodes: FullInboundStatistic


class GetFullInboundsResponseDto(RootModel[List[FullInboundResponseDto]]):
    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]
    
    def __bool__(self):
        """Return True if list is not empty"""
        return bool(self.root)
    
    def __len__(self):
        """Return length of list"""
        return len(self.root)


# Legacy aliases for backward compatibility
InboundsResponseDto = GetInboundsResponseDto
FullInboundsResponseDto = GetFullInboundsResponseDto



from typing import Annotated

from rapid_api_client.annotations import Path, PydanticBody

from remnawave.enums import TemplateType
from remnawave.models import (
    CreateSubscriptionTemplateRequestDto,
    CreateSubscriptionTemplateResponseDto,
    DeleteSubscriptionTemplateResponseDto,
    GetTemplateResponseDto,
    GetTemplatesResponseDto,
    ReorderSubscriptionTemplatesRequestDto,
    ReorderSubscriptionTemplatesResponseDto,
    UpdateTemplateRequestDto,
    UpdateTemplateResponseDto,
)
from remnawave.rapid import BaseController, delete, get, patch, post


class SubscriptionsTemplateController(BaseController):
    @get("/subscription-templates", response_class=GetTemplatesResponseDto)
    async def get_all_templates(self) -> GetTemplatesResponseDto:
        """Get all subscription templates (without content)"""
        ...

    @post("/subscription-templates", response_class=CreateSubscriptionTemplateResponseDto)
    async def create_template(
        self,
        body: Annotated[CreateSubscriptionTemplateRequestDto, PydanticBody()],
    ) -> CreateSubscriptionTemplateResponseDto:
        """Create subscription template"""
        ...

    @patch("/subscription-templates", response_class=UpdateTemplateResponseDto)
    async def update_template(
        self,
        body: Annotated[UpdateTemplateRequestDto, PydanticBody()],
    ) -> UpdateTemplateResponseDto:
        """Update subscription template"""
        ...

    @get("/subscription-templates/{uuid}", response_class=GetTemplateResponseDto)
    async def get_template_by_uuid(
        self,
        uuid: Annotated[str, Path(description="Template UUID")],
    ) -> GetTemplateResponseDto:
        """Get subscription template by uuid"""
        ...

    @delete("/subscription-templates/{uuid}", response_class=DeleteSubscriptionTemplateResponseDto)
    async def delete_template(
        self,
        uuid: Annotated[str, Path(description="Template UUID")],
    ) -> DeleteSubscriptionTemplateResponseDto:
        """Delete subscription template"""
        ...
    @post("/subscription-templates/actions/reorder", response_class=ReorderSubscriptionTemplatesResponseDto)
    async def reorder_templates(
        self,
        body: Annotated[ReorderSubscriptionTemplatesRequestDto, PydanticBody()],
    ) -> ReorderSubscriptionTemplatesResponseDto:
        """Reorder subscription templates"""
        ...


import hmac
import hashlib
import json
from typing import Union, Optional

from remnawave.models.webhook import (
    WebhookPayloadDto,
    UserDto,
    NodeDto,
    HwidUserDeviceDto,
    LoginAttemptDto,
    UserHwidDeviceEventDto,
)

class WebhookHeadersDto:
    """Helper class for webhook headers"""
    
    def __init__(self, signature: str, timestamp: str):
        self.signature = signature
        self.timestamp = timestamp
    
    @classmethod
    def from_headers(cls, headers: dict[str, str]) -> "WebhookHeadersDto":
        """
        Create WebhookHeadersDto from headers dictionary.
        Handles case-insensitive header names.
        """
        signature = None
        timestamp = None
        
        for key, value in headers.items():
            lower_key = key.lower()
            if lower_key == "x-remnawave-signature":
                signature = value
            elif lower_key == "x-remnawave-timestamp":
                timestamp = value
        
        if not signature or not timestamp:
            raise ValueError("Missing required webhook headers")
        
        return cls(signature=signature, timestamp=timestamp)


class WebhookUtility:
    @staticmethod
    def validate_webhook(
        body: Union[str, dict],
        signature: str,
        webhook_secret: str
    ) -> bool:
        """
        Validates the webhook's authenticity using HMAC SHA-256.

        :param body: The webhook request body (either a JSON string or a parsed dictionary).
        :param signature: The signature received from the server.
        :param webhook_secret: The secret key used to compute the HMAC.
        :return: True if the signature matches, otherwise False.
        """
        if isinstance(body, str):
            original_body = body
        else:
            original_body = json.dumps(body, separators=(',', ':'))

        computed_signature = hmac.new(
            webhook_secret.encode('utf-8'),
            original_body.encode('utf-8'),
            hashlib.sha256
        ).hexdigest()

        return hmac.compare_digest(computed_signature, signature)

    @staticmethod
    def validate_webhook_with_headers(
        body: Union[str, dict],
        headers: Union[dict[str, str], WebhookHeadersDto],
        webhook_secret: str
    ) -> bool:
        """
        Validates the webhook using headers object.

        :param body: The webhook request body.
        :param headers: Dictionary with headers or WebhookHeadersDto object.
        :param webhook_secret: The secret key used to compute the HMAC.
        :return: True if the signature matches, otherwise False.
        """
        if isinstance(headers, dict):
            headers = WebhookHeadersDto.from_headers(headers)
        
        return WebhookUtility.validate_webhook(body, headers.signature, webhook_secret)

    @staticmethod
    def parse_webhook(
        body: Union[str, dict],
        headers: Union[dict[str, str], WebhookHeadersDto],
        webhook_secret: str,
        validate: bool = True
    ) -> Optional[WebhookPayloadDto]:
        """
        Parses and optionally validates the webhook payload.

        :param body: The webhook request body.
        :param headers: Dictionary with headers or WebhookHeadersDto object.
        :param webhook_secret: The secret key used to compute the HMAC.
        :param validate: Whether to validate the webhook signature (default: True).
        :return: Parsed WebhookPayloadDto or None if validation fails.
        """
        if validate and not WebhookUtility.validate_webhook_with_headers(body, headers, webhook_secret):
            return None

        if isinstance(body, str):
            body = json.loads(body)

        return WebhookPayloadDto.from_dict(body)

    @staticmethod
    def is_user_event(event: str) -> bool:
        """Check if event is a user event."""
        return event.startswith("user.")

    @staticmethod
    def is_user_hwid_devices_event(event: str) -> bool:
        """Check if event is a user HWID devices event."""
        return event.startswith("user_hwid_devices.")

    @staticmethod
    def is_node_event(event: str) -> bool:
        """Check if event is a node event."""
        return event.startswith("node.")

    @staticmethod
    def is_infra_billing_event(event: str) -> bool:
        """Check if event is an infra billing event."""
        return event.startswith("crm.infra_billing")

    @staticmethod
    def is_crm_event(event: str) -> bool:
        """Check if event is a CRM event."""
        return event.startswith("crm.")

    @staticmethod
    def is_service_event(event: str) -> bool:
        """Check if event is a service event."""
        return event.startswith("service.")

    @staticmethod
    def is_errors_event(event: str) -> bool:
        """Check if event is an errors event."""
        return event.startswith("errors.")

    @staticmethod
    def get_typed_data(payload: WebhookPayloadDto) -> Union[UserDto, NodeDto, HwidUserDeviceDto, LoginAttemptDto, UserHwidDeviceEventDto, dict]:
        """
        Get typed data from webhook payload based on event type.
        
        :param payload: Parsed webhook payload.
        :return: Typed data object.
        """
        return payload.data

    @staticmethod
    def extract_user_hwid_event_data(payload: WebhookPayloadDto) -> Optional[tuple[UserDto, HwidUserDeviceDto]]:
        """
        Extract user and HWID device from user_hwid_devices event.
        
        :param payload: Parsed webhook payload.
        :return: Tuple of (UserDto, HwidUserDeviceDto) or None if not a HWID event.
        """
        if not WebhookUtility.is_user_hwid_devices_event(payload.event):
            return None
        
        if isinstance(payload.data, dict):
            user_data = payload.data.get("user", {})
            hwid_data = payload.data.get("hwidUserDevice", {})
            
            return (
                UserDto(**user_data),
                HwidUserDeviceDto(**hwid_data)
            )
        
        return None


from datetime import datetime
from enum import StrEnum
from typing import Dict, List, Optional
from uuid import UUID

from pydantic import BaseModel, Field
from remnawave.models import CustomRemarksDto, HwidSettingsDto


class TemplateType(StrEnum):
    """Template type enum"""
    XRAY_JSON = "XRAY_JSON"
    XRAY_BASE64 = "XRAY_BASE64"
    MIHOMO = "MIHOMO"
    STASH = "STASH"
    CLASH = "CLASH"
    SINGBOX = "SINGBOX"


class ExternalSquadInfoDto(BaseModel):
    """External squad info"""
    members_count: float = Field(alias="membersCount")


class ExternalSquadTemplateDto(BaseModel):
    """External squad template"""
    template_uuid: UUID = Field(alias="templateUuid")
    template_type: TemplateType = Field(alias="templateType")


class ExternalSquadSubscriptionSettingsDto(BaseModel):
    """External squad subscription settings"""
    profile_title: Optional[str] = Field(None, alias="profileTitle")
    support_link: Optional[str] = Field(None, alias="supportLink")
    profile_update_interval: Optional[int] = Field(None, alias="profileUpdateInterval", ge=1)
    is_profile_webpage_url_enabled: Optional[bool] = Field(None, alias="isProfileWebpageUrlEnabled")
    serve_json_at_base_subscription: Optional[bool] = Field(None, alias="serveJsonAtBaseSubscription")
    is_show_custom_remarks: Optional[bool] = Field(None, alias="isShowCustomRemarks")
    happ_announce: Optional[str] = Field(None, alias="happAnnounce")
    happ_routing: Optional[str] = Field(None, alias="happRouting")
    randomize_hosts: Optional[bool] = Field(None, alias="randomizeHosts")

class ExternalSquadHostOverridesDto(BaseModel):
    """External squad host overrides"""
    server_description: Optional[str] = Field(None, alias="serverDescription", max_length=30)
    vless_route_id: Optional[int] = Field(None, alias="vlessRouteId", ge=0, le=65535)


class ExternalSquadDto(BaseModel):
    """External squad data model"""
    uuid: UUID
    view_position: int = Field(alias="viewPosition")
    name: str
    info: ExternalSquadInfoDto
    templates: List[ExternalSquadTemplateDto]
    subscription_settings: Optional[ExternalSquadSubscriptionSettingsDto] = Field(None, alias="subscriptionSettings")
    host_overrides: Optional[ExternalSquadHostOverridesDto] = Field(None, alias="hostOverrides")
    response_headers: Optional[Dict[str, str]] = Field(None, alias="responseHeaders")
    hwid_settings: Optional[HwidSettingsDto] = Field(None, alias="hwidSettings")
    custom_remarks: Optional[CustomRemarksDto] = Field(None, alias="customRemarks")
    created_at: datetime = Field(alias="createdAt")
    updated_at: datetime = Field(alias="updatedAt")


# Request/Response models
class GetExternalSquadsResponseDto(BaseModel):
    """Response with all external squads"""
    total: int = Field(alias="total")
    external_squads: List[ExternalSquadDto] = Field(alias="externalSquads")


class GetExternalSquadByUuidResponseDto(ExternalSquadDto):
    """Response with external squad by UUID"""
    pass


class CreateExternalSquadRequestDto(BaseModel):
    """Request to create external squad"""
    name: str = Field(min_length=2, max_length=30, pattern=r"^[A-Za-z0-9_\s-]+$")


class CreateExternalSquadResponseDto(ExternalSquadDto):
    """Response after creating external squad"""
    pass


class UpdateExternalSquadRequestDto(BaseModel):
    """Request to update external squad"""
    uuid: UUID
    name: Optional[str] = Field(None, min_length=2, max_length=30, pattern=r"^[A-Za-z0-9_\s-]+$")
    templates: Optional[List[ExternalSquadTemplateDto]] = None
    subscription_settings: Optional[ExternalSquadSubscriptionSettingsDto] = Field(None, serialization_alias="subscriptionSettings")
    host_overrides: Optional[ExternalSquadHostOverridesDto] = Field(None, serialization_alias="hostOverrides")
    hwid_settings: Optional[HwidSettingsDto] = Field(None, alias="hwidSettings")
    custom_remarks: Optional[CustomRemarksDto] = Field(None, alias="customRemarks")
    response_headers: Optional[Dict[str, str]] = Field(None, serialization_alias="responseHeaders")


class UpdateExternalSquadResponseDto(ExternalSquadDto):
    """Response after updating external squad"""
    pass


class DeleteExternalSquadResponseDto(BaseModel):
    """Response after deleting external squad"""
    is_deleted: bool = Field(alias="isDeleted")


class ReorderExternalSquadItem(BaseModel):
    view_position: int = Field(serialization_alias="viewPosition")
    uuid: UUID


class ReorderExternalSquadsRequestDto(BaseModel):
    items: List[ReorderExternalSquadItem]


class ReorderExternalSquadsResponseDto(BaseModel):
    """Response after reordering external squads"""
    total: int = Field(alias="total")
    external_squads: List[ExternalSquadDto] = Field(alias="externalSquads")


class DeleteExternalSquadResponseDto(BaseModel):
    """Response after deleting external squad"""
    is_deleted: bool = Field(alias="isDeleted")


class AddUsersToExternalSquadResponseDto(BaseModel):
    """Response after adding users to external squad"""
    event_sent: bool = Field(alias="eventSent")


class RemoveUsersFromExternalSquadResponseDto(BaseModel):
    """Response after removing users from external squad"""
    event_sent: bool = Field(alias="eventSent")


from typing import Annotated, List, Literal, Optional, Union
from uuid import UUID

from pydantic import BaseModel, Field


# ─────────────────────────────────────────────────────────────────────────────
# Fetch IPs – step 1: start the job
# ─────────────────────────────────────────────────────────────────────────────

class FetchIpsJobData(BaseModel):
    """Returned job ID after requesting IP fetch"""
    job_id: str = Field(alias="jobId")


class FetchIpsResponseDto(FetchIpsJobData):
    """Response for POST /api/ip-control/fetch-ips/{uuid}"""
    pass


# ─────────────────────────────────────────────────────────────────────────────
# Fetch IPs – step 2: poll the job result
# ─────────────────────────────────────────────────────────────────────────────

class FetchIpsProgressData(BaseModel):
    """Progress information for an IP-fetch job"""
    total: int
    completed: int
    percent: float


class FetchIpsNodeResult(BaseModel):
    """Per-node IP list for a user"""
    node_uuid: UUID = Field(alias="nodeUuid")
    node_name: str = Field(alias="nodeName")
    country_code: str = Field(alias="countryCode")
    ips: List[str]


class FetchIpsResult(BaseModel):
    """Full result payload when the job is completed"""
    success: bool
    user_uuid: UUID = Field(alias="userUuid")
    user_id: str = Field(alias="userId")
    nodes: List[FetchIpsNodeResult]


class FetchIpsResultData(BaseModel):
    """Job state + optional result"""
    is_completed: bool = Field(alias="isCompleted")
    is_failed: bool = Field(alias="isFailed")
    progress: FetchIpsProgressData
    result: Optional[FetchIpsResult] = None


class FetchIpsResultResponseDto(FetchIpsResultData):
    """Response for GET /api/ip-control/fetch-ips/result/{jobId}"""
    pass


# ─────────────────────────────────────────────────────────────────────────────
# Drop Connections request – discriminated unions for dropBy / targetNodes
# ─────────────────────────────────────────────────────────────────────────────

class DropByUserUuids(BaseModel):
    """Drop connections for specific user UUIDs"""
    by: Literal["userUuids"] = "userUuids"
    user_uuids: List[UUID] = Field(
        ...,
        serialization_alias="userUuids",
        min_length=1,
        description="List of user UUIDs whose connections should be dropped",
    )


class DropByIpAddresses(BaseModel):
    """Drop connections from specific IP addresses"""
    by: Literal["ipAddresses"] = "ipAddresses"
    ip_addresses: List[str] = Field(
        ...,
        serialization_alias="ipAddresses",
        min_length=1,
        description="List of IP addresses to disconnect",
    )


# Discriminated union – use `by` field as the discriminator
DropBy = Annotated[
    Union[DropByUserUuids, DropByIpAddresses],
    Field(discriminator="by"),
]


class TargetAllNodes(BaseModel):
    """Send the drop-connections event to all connected nodes"""
    target: Literal["allNodes"] = "allNodes"


class TargetSpecificNodes(BaseModel):
    """Send the drop-connections event to specific nodes only"""
    target: Literal["specificNodes"] = "specificNodes"
    node_uuids: List[UUID] = Field(
        ...,
        serialization_alias="nodeUuids",
        min_length=1,
        description="List of node UUIDs to target",
    )


# Discriminated union – use `target` field as the discriminator
TargetNodes = Annotated[
    Union[TargetAllNodes, TargetSpecificNodes],
    Field(discriminator="target"),
]


class DropConnectionsRequestDto(BaseModel):
    """Request body for POST /api/ip-control/drop-connections"""
    drop_by: DropBy = Field(
        ...,
        serialization_alias="dropBy",
        description="Selector for whose connections to drop",
    )
    target_nodes: TargetNodes = Field(
        ...,
        serialization_alias="targetNodes",
        description="Selector for which nodes to send the drop event to",
    )


# ─────────────────────────────────────────────────────────────────────────────
# Drop Connections response
# ─────────────────────────────────────────────────────────────────────────────

class DropConnectionsResponseData(BaseModel):
    """Payload confirming the drop-connections event was sent"""
    event_sent: bool = Field(alias="eventSent")


class DropConnectionsResponseDto(DropConnectionsResponseData):
    """Response for POST /api/ip-control/drop-connections"""
    pass



from typing import Annotated

from rapid_api_client import Path, Query

from remnawave.models.bandwidthstats import (
    GetLegacyStatsNodesUsersUsageResponseDto,
    GetLegacyStatsUserUsageResponseDto,
    GetNodeUserUsageByRangeResponseDto,
    GetNodesRealtimeUsageResponseDto,
    GetNodesUsageByRangeResponseDto,
    GetStatsNodesRealtimeUsageResponseDto,
    GetStatsNodesUsageResponseDto,
    GetStatsNodeUsersUsageResponseDto,
    GetStatsUserUsageResponseDto,
    GetUserUsageByRangeResponseDto,
)
from remnawave.rapid import BaseController, get


class BandWidthStatsController(BaseController):
    # ============ Legacy Endpoints (Deprecated) ============

    @get("/bandwidth-stats/users/{userUuid}/legacy", response_class=GetUserUsageByRangeResponseDto)
    async def get_user_usage_legacy_old(
        self,
        user_uuid: Annotated[str, Path(description="UUID of the user", alias="userUuid")],
        start: Annotated[str, Query(description="Start date")],
        end: Annotated[str, Query(description="End date")],
    ) -> GetUserUsageByRangeResponseDto:
        """Get User Usage by Range (Legacy - Deprecated)"""
        ...

    @get("/bandwidth-stats/nodes/{nodeUuid}/users/legacy", response_class=GetNodeUserUsageByRangeResponseDto)
    async def get_node_user_usage_legacy_old(
        self,
        node_uuid: Annotated[str, Path(description="UUID of the node", alias="nodeUuid")],
        start: Annotated[str, Query(description="Start date")],
        end: Annotated[str, Query(description="End date")],
    ) -> GetNodeUserUsageByRangeResponseDto:
        """Get Node User Usage by Range and Node UUID (Legacy - Deprecated)"""
        ...

    # ============ New Stats Endpoints ============
    
    @get("/bandwidth-stats/nodes/realtime", response_class=GetStatsNodesRealtimeUsageResponseDto)
    async def get_nodes_realtime_usage(
        self,
    ) -> GetStatsNodesRealtimeUsageResponseDto:
        """Get Nodes Realtime Usage"""
        ...

    @get("/bandwidth-stats/nodes/{uuid}/users/legacy", response_class=GetLegacyStatsNodesUsersUsageResponseDto)
    async def get_node_users_usage_legacy_stats(
        self,
        uuid: Annotated[str, Path(description="UUID of the node")],
        start: Annotated[str, Query(description="Start date")],
        end: Annotated[str, Query(description="End date")],
    ) -> GetLegacyStatsNodesUsersUsageResponseDto:
        """Get Node Users Usage by Range and Node UUID (Legacy Stats)"""
        ...

    @get("/bandwidth-stats/nodes/{uuid}/users", response_class=GetStatsNodeUsersUsageResponseDto)
    async def get_stats_node_users_usage(
        self,
        uuid: Annotated[str, Path(description="UUID of the node")],
        top_users_limit: Annotated[int, Query(description="Limit of top users to return", alias="topUsersLimit")],
        start: Annotated[str, Query(description="Start date")],
        end: Annotated[str, Query(description="End date")],
    ) -> GetStatsNodeUsersUsageResponseDto:
        """Get Node Users Usage by Node UUID"""
        ...

    @get("/bandwidth-stats/users/{uuid}", response_class=GetStatsUserUsageResponseDto)
    async def get_stats_user_usage(
        self,
        uuid: Annotated[str, Path(description="UUID of the user")],
        top_nodes_limit: Annotated[int, Query(description="Limit of top nodes to return", alias="topNodesLimit")],
        start: Annotated[str, Query(description="Start date")],
        end: Annotated[str, Query(description="End date")],
    ) -> GetStatsUserUsageResponseDto:
        """Get User Usage by Range"""
        ...

    @get("/bandwidth-stats/nodes", response_class=GetStatsNodesUsageResponseDto)
    async def get_stats_nodes_usage(
        self,
        top_nodes_limit: Annotated[int, Query(description="Limit of top nodes to return", alias="topNodesLimit")],
        start: Annotated[str, Query(description="Start date")],
        end: Annotated[str, Query(description="End date")],
    ) -> GetStatsNodesUsageResponseDto:
        """Get Nodes Usage by Range"""
        ...

    @get("/bandwidth-stats/users/{uuid}/legacy", response_class=GetLegacyStatsUserUsageResponseDto)
    async def get_user_usage_legacy_stats(
        self,
        uuid: Annotated[str, Path(description="UUID of the user")],
        start: Annotated[str, Query(description="Start date")],
        end: Annotated[str, Query(description="End date")],
    ) -> GetLegacyStatsUserUsageResponseDto:
        """Get User Usage by Range (Legacy Stats)"""
        ...


from datetime import datetime
from typing import Annotated, Dict, List, Optional
from uuid import UUID

from pydantic import BaseModel, Field, StringConstraints

from remnawave.enums import (
    ResponseRuleConditionOperator,
    ResponseRuleOperator,
    ResponseRuleVersion,
    ResponseType,
    SubscriptionType,
)


class ResponseRuleCondition(BaseModel):
    """Condition to check against request headers"""
    header_name: Annotated[str, StringConstraints(pattern=r"^[!#$%&'*+\-.0-9A-Z^_`a-z|~]+$")] = Field(
        alias="headerName"
    )
    operator: ResponseRuleConditionOperator
    value: Annotated[str, StringConstraints(min_length=1, max_length=255)]
    case_sensitive: bool = Field(alias="caseSensitive")


class ResponseModificationHeader(BaseModel):
    """Response header modification"""
    key: Annotated[str, StringConstraints(pattern=r"^[!#$%&'*+\-.0-9A-Z^_`a-z|~]+$")]
    value: Annotated[str, StringConstraints(min_length=1)]


class ResponseModifications(BaseModel):
    """Response modifications to apply when rule matches"""
    headers: Optional[List[ResponseModificationHeader]] = None
    apply_headers_to_end: Optional[bool] = Field(
        None,
        alias="applyHeadersToEnd",
        description=(
            "If True, SRR headers are appended at the very end of the response "
            "and may override headers from other sections."
        ),
    )
    subscription_template: Optional[Annotated[str, StringConstraints(min_length=1)]] = Field(
        None, alias="subscriptionTemplate"
    )
    ignore_host_xray_json_template: Optional[bool] = Field(
        None,
        alias="ignoreHostXrayJsonTemplate",
        description=(
            "If True, the Host's own Xray Json Template is ignored and the "
            "template defined by the SRR rule is used instead."
        ),
    )
    ignore_serve_json_at_base_subscription: Optional[bool] = Field(
        None,
        alias="ignoreServeJsonAtBaseSubscription",
        description=(
            "If True, the Serve JSON at Base Subscription setting is ignored "
            "(treated as False)."
        ),
    )


class ResponseRule(BaseModel):
    """Individual response rule configuration"""
    name: Annotated[str, StringConstraints(min_length=1, max_length=50)]
    description: Optional[Annotated[str, StringConstraints(min_length=1, max_length=250)]] = None
    enabled: bool
    operator: ResponseRuleOperator
    conditions: List[ResponseRuleCondition]
    response_type: ResponseType = Field(alias="responseType")
    response_modifications: Optional[ResponseModifications] = Field(
        None, alias="responseModifications"
    )


class ResponseRules(BaseModel):
    """Response rules configuration"""
    version: ResponseRuleVersion
    rules: List[ResponseRule]


class CustomRemarksDto(BaseModel):
    """Custom remarks for different user states"""
    expired_users: List[str] = Field(alias="expiredUsers", min_length=1)
    limited_users: List[str] = Field(alias="limitedUsers", min_length=1)
    disabled_users: List[str] = Field(alias="disabledUsers", min_length=1)
    empty_hosts: List[str] = Field(alias="emptyHosts", min_length=1)
    hwid_max_devices_exceeded: List[str] = Field(alias="HWIDMaxDevicesExceeded", min_length=1)
    hwid_not_supported: List[str] = Field(alias="HWIDNotSupported", min_length=1)


class HwidSettingsDto(BaseModel):
    """HWID (Hardware ID) settings"""
    enabled: bool
    fallback_device_limit: int = Field(alias="fallbackDeviceLimit")
    max_devices_announce: Optional[Annotated[str, StringConstraints(max_length=200)]] = Field(
        None, alias="maxDevicesAnnounce"
    )

class SubscriptionSettingsResponseDto(BaseModel):
    """Subscription settings response data"""
    uuid: UUID
    profile_title: str = Field(alias="profileTitle")
    support_link: str = Field(alias="supportLink")
    profile_update_interval: int = Field(alias="profileUpdateInterval", ge=1)
    is_profile_webpage_url_enabled: bool = Field(alias="isProfileWebpageUrlEnabled")
    serve_json_at_base_subscription: bool = Field(alias="serveJsonAtBaseSubscription")
    show_custom_remarks: bool = Field(alias="isShowCustomRemarks")
    
    custom_remarks: CustomRemarksDto = Field(alias="customRemarks")
    
    happ_announce: Optional[str] = Field(None, alias="happAnnounce")
    happ_routing: Optional[str] = Field(None, alias="happRouting")
    custom_response_headers: Optional[Dict[str, str]] = Field(None, alias="customResponseHeaders")
    randomize_hosts: bool = Field(alias="randomizeHosts")
    response_rules: Optional[ResponseRules] = Field(None, alias="responseRules")
    
    hwid_settings: Optional[HwidSettingsDto] = Field(None, alias="hwidSettings")
    
    created_at: datetime = Field(alias="createdAt")
    updated_at: datetime = Field(alias="updatedAt")


class GetSubscriptionSettingsResponseDto(SubscriptionSettingsResponseDto):
    pass


class UpdateSubscriptionSettingsResponseDto(SubscriptionSettingsResponseDto):
    pass


class UpdateSubscriptionSettingsRequestDto(BaseModel):
    """Update subscription settings request"""
    uuid: UUID
    profile_title: Optional[str] = Field(None, serialization_alias="profileTitle")
    support_link: Optional[str] = Field(None, serialization_alias="supportLink")
    profile_update_interval: Optional[int] = Field(None, serialization_alias="profileUpdateInterval")
    is_profile_webpage_url_enabled: Optional[bool] = Field(
        None, serialization_alias="isProfileWebpageUrlEnabled"
    )
    serve_json_at_base_subscription: Optional[bool] = Field(
        None, serialization_alias="serveJsonAtBaseSubscription"
    )
    is_show_custom_remarks: Optional[bool] = Field(None, serialization_alias="isShowCustomRemarks")
    
    custom_remarks: Optional[CustomRemarksDto] = Field(None, serialization_alias="customRemarks")
    
    happ_announce: Optional[Annotated[str, StringConstraints(max_length=200)]] = Field(
        None, serialization_alias="happAnnounce"
    )
    happ_routing: Optional[str] = Field(None, serialization_alias="happRouting")
    custom_response_headers: Optional[Dict[str, str]] = Field(
        None, serialization_alias="customResponseHeaders"
    )
    randomize_hosts: Optional[bool] = Field(None, serialization_alias="randomizeHosts")
    response_rules: Optional[ResponseRules] = Field(None, serialization_alias="responseRules")
    
    hwid_settings: Optional[HwidSettingsDto] = Field(None, serialization_alias="hwidSettings")


# Backward compatibility aliases
CustomRemarks = CustomRemarksDto
HwidSettings = HwidSettingsDto


from typing import Annotated

from rapid_api_client.annotations import PydanticBody

from remnawave.models import (
    AddUsersToExternalSquadResponseDto,
    CreateExternalSquadRequestDto,
    CreateExternalSquadResponseDto,
    DeleteExternalSquadResponseDto,
    GetExternalSquadByUuidResponseDto,
    GetExternalSquadsResponseDto,
    RemoveUsersFromExternalSquadResponseDto,
    ReorderExternalSquadsRequestDto,
    ReorderExternalSquadsResponseDto,
    UpdateExternalSquadRequestDto,
    UpdateExternalSquadResponseDto,
)
from remnawave.rapid import BaseController, delete, get, patch, post


class ExternalSquadsController(BaseController):
    @get("/external-squads", response_class=GetExternalSquadsResponseDto)
    async def get_external_squads(
        self,
    ) -> GetExternalSquadsResponseDto:
        """Get all external squads"""
        ...

    @post("/external-squads", response_class=CreateExternalSquadResponseDto)
    async def create_external_squad(
        self,
        body: Annotated[CreateExternalSquadRequestDto, PydanticBody()],
    ) -> CreateExternalSquadResponseDto:
        """Create external squad"""
        ...

    @patch("/external-squads", response_class=UpdateExternalSquadResponseDto)
    async def update_external_squad(
        self,
        body: Annotated[UpdateExternalSquadRequestDto, PydanticBody()],
    ) -> UpdateExternalSquadResponseDto:
        """Update external squad"""
        ...

    @get("/external-squads/{uuid}", response_class=GetExternalSquadByUuidResponseDto)
    async def get_external_squad_by_uuid(
        self,
        uuid: str,
    ) -> GetExternalSquadByUuidResponseDto:
        """Get external squad by uuid"""
        ...

    @delete("/external-squads/{uuid}", response_class=DeleteExternalSquadResponseDto)
    async def delete_external_squad(
        self,
        uuid: str,
    ) -> DeleteExternalSquadResponseDto:
        """Delete external squad"""
        ...

    @post("/external-squads/{uuid}/bulk-actions/add-users", response_class=AddUsersToExternalSquadResponseDto)
    async def add_users_to_external_squad(
        self,
        uuid: str,
    ) -> AddUsersToExternalSquadResponseDto:
        """Add all users to external squad"""
        ...

    @delete("/external-squads/{uuid}/bulk-actions/remove-users", response_class=RemoveUsersFromExternalSquadResponseDto)
    async def remove_users_from_external_squad(
        self,
        uuid: str,
    ) -> RemoveUsersFromExternalSquadResponseDto:
        """Delete users from external squad"""
        ...
    @post("/external-squads/actions/reorder", response_class=ReorderExternalSquadsResponseDto)
    async def reorder_external_squads(
        self,
        body: Annotated[ReorderExternalSquadsRequestDto, PydanticBody()],
    ) -> ReorderExternalSquadsResponseDto:
        """Reorder external squads"""
        ...


from datetime import datetime
from typing import Annotated, List, Literal, Optional
from uuid import UUID

from pydantic import (
    BaseModel,
    EmailStr,
    Field,
    RootModel,
    StringConstraints,
)

from remnawave.enums import TrafficLimitStrategy, UserStatus
from remnawave.utils.happ_crypt import create_happ_crypto_link


class UserActiveInboundsDto(BaseModel):
    uuid: UUID
    tag: str
    type: str
    network: str | None = None
    security: str | None = None


class UserLastConnectedNodeDto(BaseModel):
    connected_at: datetime = Field(alias="connectedAt")
    node_name: str = Field(alias="nodeName")


class ActiveInternalSquadDto(BaseModel):
    uuid: UUID
    name: str


class HappCrypto(BaseModel):
    crypto_link: str = Field(alias="cryptoLink")


class CreateUserRequestDto(BaseModel):
    """Request DTO for creating a user"""
    # Required fields
    username: Annotated[
        str, 
        StringConstraints(pattern=r"^[a-zA-Z0-9_-]+$", min_length=3, max_length=36)
    ] = Field(..., description="Unique username for the user")
    expire_at: datetime = Field(..., serialization_alias="expireAt", description="Account expiration date")
    
    # Optional fields with defaults
    status: UserStatus = Field(default=UserStatus.ACTIVE, description="User account status")
    traffic_limit_strategy: TrafficLimitStrategy = Field(
        default=TrafficLimitStrategy.NO_RESET,
        serialization_alias="trafficLimitStrategy",
        description="Traffic reset strategy"
    )
    
    # Optional fields
    short_uuid: Optional[str] = Field(None, serialization_alias="shortUuid")
    trojan_password: Optional[Annotated[str, StringConstraints(min_length=8, max_length=32)]] = Field(
        None, serialization_alias="trojanPassword"
    )
    vless_uuid: Optional[UUID] = Field(None, serialization_alias="vlessUuid")
    ss_password: Optional[Annotated[str, StringConstraints(min_length=8, max_length=32)]] = Field(
        None, serialization_alias="ssPassword"
    )
    traffic_limit_bytes: Optional[int] = Field(
        None, serialization_alias="trafficLimitBytes", ge=0
    )
    created_at: Optional[datetime] = Field(None, serialization_alias="createdAt")
    last_traffic_reset_at: Optional[datetime] = Field(None, serialization_alias="lastTrafficResetAt")
    description: Optional[str] = None
    tag: Optional[Annotated[str, StringConstraints(max_length=16, pattern=r"^[A-Z0-9_]+$")]] = None
    telegram_id: Optional[int] = Field(None, serialization_alias="telegramId")
    email: Optional[EmailStr] = None
    hwid_device_limit: Optional[int] = Field(None, serialization_alias="hwidDeviceLimit", ge=0)
    active_internal_squads: Optional[List[UUID]] = Field(None, serialization_alias="activeInternalSquads")
    uuid: Optional[UUID] = Field(
        None, 
        description="Optional. Pass UUID to create user with specific UUID, otherwise it will be generated automatically."
    )
    external_squad_uuid: Optional[UUID] = Field(None, serialization_alias="externalSquadUuid")


class UpdateUserRequestDto(BaseModel):
    """Request DTO for updating a user"""
    # Either username or uuid must be provided, uuid has priority
    username: Optional[str] = Field(None, description="Username of the user")
    uuid: Optional[UUID] = Field(
        None, 
        description="UUID of the user. UUID has higher priority than username"
    )
    
    # Optional update fields
    status: Optional[UserStatus] = None
    description: Optional[str] = None
    email: Optional[EmailStr] = None
    expire_at: Optional[datetime] = Field(None, serialization_alias="expireAt")
    hwid_device_limit: Optional[int] = Field(None, serialization_alias="hwidDeviceLimit", ge=0)
    tag: Optional[Annotated[str, StringConstraints(max_length=16, pattern=r"^[A-Z0-9_]+$")]] = None
    telegram_id: Optional[int] = Field(None, serialization_alias="telegramId")
    traffic_limit_bytes: Optional[int] = Field(None, serialization_alias="trafficLimitBytes", ge=0)
    traffic_limit_strategy: Optional[TrafficLimitStrategy] = Field(
        None, serialization_alias="trafficLimitStrategy"
    )
    active_internal_squads: Optional[List[UUID]] = Field(
        None, serialization_alias="activeInternalSquads"
    )
    external_squad_uuid: Optional[UUID] = Field(None, serialization_alias="externalSquadUuid")


class UserTrafficDto(BaseModel):
    """User traffic information"""
    used_traffic_bytes: int = Field(alias="usedTrafficBytes")
    lifetime_used_traffic_bytes: int = Field(alias="lifetimeUsedTrafficBytes")
    online_at: Optional[datetime] = Field(None, alias="onlineAt")
    first_connected_at: Optional[datetime] = Field(None, alias="firstConnectedAt")
    last_connected_node_uuid: Optional[UUID] = Field(None, alias="lastConnectedNodeUuid")


class UserResponseDto(BaseModel):
    """User response DTO - обновленная структура с userTraffic"""
    uuid: UUID
    id: int
    short_uuid: str = Field(alias="shortUuid")
    username: str
    status: UserStatus = Field(default=UserStatus.ACTIVE)
    traffic_limit_bytes: int = Field(0, alias="trafficLimitBytes")
    traffic_limit_strategy: TrafficLimitStrategy = Field(
        TrafficLimitStrategy.NO_RESET, alias="trafficLimitStrategy"
    )
    expire_at: datetime = Field(alias="expireAt")
    telegram_id: Optional[int] = Field(None, alias="telegramId")
    email: Optional[str] = None
    description: Optional[str] = None
    tag: Optional[str] = None
    hwid_device_limit: Optional[int] = Field(None, alias="hwidDeviceLimit")
    external_squad_uuid: Optional[UUID] = Field(None, alias="externalSquadUuid")
    trojan_password: str = Field(alias="trojanPassword")
    vless_uuid: UUID = Field(alias="vlessUuid")
    ss_password: str = Field(alias="ssPassword")
    last_trigger_threshold: int = Field(0, alias="lastTriggeredThreshold")
    sub_revoked_at: Optional[datetime] = Field(None, alias="subRevokedAt")
    sub_last_user_agent: Optional[str] = Field(None, alias="subLastUserAgent")
    sub_last_opened_at: Optional[datetime] = Field(None, alias="subLastOpenedAt")
    last_traffic_reset_at: Optional[datetime] = Field(None, alias="lastTrafficResetAt")
    created_at: datetime = Field(alias="createdAt")
    updated_at: datetime = Field(alias="updatedAt")
    subscription_url: str = Field(alias="subscriptionUrl")
    active_internal_squads: list[ActiveInternalSquadDto] = Field(alias="activeInternalSquads")
    user_traffic: UserTrafficDto = Field(alias="userTraffic")
    
    @property
    def used_traffic_bytes(self) -> int:
        """Backward compatibility property"""
        return self.user_traffic.used_traffic_bytes
    
    @property
    def lifetime_used_traffic_bytes(self) -> int:
        """Backward compatibility property"""
        return self.user_traffic.lifetime_used_traffic_bytes
    
    @property
    def online_at(self) -> Optional[datetime]:
        """Backward compatibility property"""
        return self.user_traffic.online_at
    
    @property
    def first_connected(self) -> Optional[datetime]:
        """Backward compatibility property"""
        return self.user_traffic.first_connected_at
    
    @property
    def last_connected_node_uuid(self) -> Optional[UUID]:
        """Backward compatibility property"""
        return self.user_traffic.last_connected_node_uuid
    
    @property
    def happ(self) -> HappCrypto:
        """Generate Happ Crypto Link"""
        crypto_link = create_happ_crypto_link(self.subscription_url)
        return HappCrypto(cryptoLink=crypto_link)
    
    def happ_with_version(self, version: Literal["v3", "v4"] = "v4") -> HappCrypto:
        return self._generate_happ(version=version)

    def _generate_happ(self, version):
        crypto_link = create_happ_crypto_link(content=self.subscription_url, method=version)
        return HappCrypto(cryptoLink=crypto_link)


class RevokeUserRequestDto(BaseModel):
    """Request DTO for revoking user subscription"""
    short_uuid: Optional[str] = Field(
        None,
        serialization_alias="shortUuid",
        description="Optional. If not provided, a new short UUID will be generated by Remnawave.",
        min_length=6,
        max_length=48,
        pattern=r"^[a-zA-Z0-9_-]+$",
    )
    revoke_only_passwords: Optional[bool] = Field(
        None,
        serialization_alias="revokeOnlyPasswords",
        description="Optional. If true, only passwords will be revoked without changing the short UUID.",
    )

class SubscriptionRequestRecord(BaseModel):
    """Subscription request history record"""
    id: int
    user_uuid: UUID = Field(alias="userUuid")
    request_at: datetime = Field(alias="requestAt")
    request_ip: Optional[str] = Field(alias="requestIp")
    user_agent: Optional[str] = Field(alias="userAgent")


class SubscriptionRequestsResponseData(BaseModel):
    """Subscription requests response data"""
    total: int
    records: List[SubscriptionRequestRecord]


class CreateUserResponseDto(UserResponseDto):
    """Response for create user"""
    pass


class UpdateUserResponseDto(UserResponseDto):
    """Response for update user"""
    pass


class GetUserByUuidResponseDto(UserResponseDto):
    """Response for get user by UUID"""
    pass


class GetUserByShortUuidResponseDto(UserResponseDto):
    """Response for get user by short UUID"""
    pass


class GetUserByUsernameResponseDto(UserResponseDto):
    """Response for get user by username"""
    pass


class GetUserByIdResponseDto(UserResponseDto):
    """Response for get user by ID"""
    pass


class DisableUserResponseDto(UserResponseDto):
    """Response for disable user"""
    pass


class EnableUserResponseDto(UserResponseDto):
    """Response for enable user"""
    pass


class ResetUserTrafficResponseDto(UserResponseDto):
    """Response for reset user traffic"""
    pass


class RevokeUserSubscriptionResponseDto(UserResponseDto):
    """Response for revoke user subscription"""
    pass


class ActivateAllInboundsResponseDto(UserResponseDto):
    """Response for activate all inbounds"""
    pass


class UsersResponseDto(BaseModel):
    """Users collection response"""
    users: list[UserResponseDto]
    total: int


class GetAllUsersResponseDto(UsersResponseDto):
    """Response for get all users"""
    pass


class TagsResponseDto(BaseModel):
    """Tags collection response"""
    tags: list[str]


class GetAllTagsResponseDto(TagsResponseDto):
    """Response for get all tags"""
    pass


class GetSubscriptionRequestsResponseDto(SubscriptionRequestsResponseData):
    """Response for get subscription requests"""
    pass


class GetUserSubscriptionRequestHistoryResponseDto(SubscriptionRequestsResponseData):
    """Response for get user subscription request history"""
    pass


class DeleteUserResponseDto(BaseModel):
    """Response for delete user"""
    is_deleted: bool = Field(alias="isDeleted")


class EmailUserResponseDto(RootModel[list[UserResponseDto]]):
    """Response for get users by email"""
    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]
    
    def __bool__(self):
        """Return True if list is not empty"""
        return bool(self.root)
    
    def __len__(self):
        """Return length of list"""
        return len(self.root)


class TagUserResponseDto(RootModel[list[UserResponseDto]]):
    """Response for get users by tag"""
    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]
    
    def __bool__(self):
        """Return True if list is not empty"""
        return bool(self.root)
    
    def __len__(self):
        """Return length of list"""
        return len(self.root)


class TelegramUserResponseDto(RootModel[list[UserResponseDto]]):
    """Response for get users by telegram ID"""
    def __iter__(self):
        return iter(self.root)

    def __getitem__(self, item):
        return self.root[item]
    
    def __bool__(self):
        """Return True if list is not empty"""
        return bool(self.root)
    
    def __len__(self):
        """Return length of list"""
        return len(self.root)


from typing import Annotated, Optional

from rapid_api_client import Path, Query
from rapid_api_client.annotations import PydanticBody

from remnawave.models import (
    CreateUserRequestDto,
    CreateUserResponseDto,
    DeleteUserResponseDto,
    GetAllUsersResponseDto,
    GetAllTagsResponseDto,
    GetUserByIdResponseDto,
    GetUserByShortUuidResponseDto,
    GetUserByUsernameResponseDto,
    GetUserByUuidResponseDto,
    GetUserAccessibleNodesResponseDto,
    GetUserSubscriptionRequestHistoryResponseDto,
    TelegramUserResponseDto,
    EmailUserResponseDto,
    TagUserResponseDto,
    UpdateUserRequestDto,
    UpdateUserResponseDto,
    RevokeUserRequestDto,
)
from remnawave.rapid import BaseController, delete, get, patch, post


class UsersController(BaseController):
    @post("/users", response_class=CreateUserResponseDto)
    async def create_user(
        self,
        body: Annotated[CreateUserRequestDto, PydanticBody()],
    ) -> CreateUserResponseDto:
        """Create a new user"""
        ...

    @patch("/users", response_class=UpdateUserResponseDto)
    async def update_user(
        self,
        body: Annotated[UpdateUserRequestDto, PydanticBody()],
    ) -> UpdateUserResponseDto:
        """Update a user by UUID or username"""
        ...

    @get("/users", response_class=GetAllUsersResponseDto)
    async def get_all_users(
        self,
        start: Annotated[
            Optional[int], 
            Query(default=None, description="Offset for pagination")
        ] = None,
        size: Annotated[
            Optional[int], 
            Query(default=None, description="Page size for pagination")
        ] = None,
    ) -> GetAllUsersResponseDto:
        """Get all users"""
        ...

    @delete("/users/{uuid}", response_class=DeleteUserResponseDto)
    async def delete_user(
        self,
        uuid: Annotated[str, Path(description="UUID of the user")],
    ) -> DeleteUserResponseDto:
        """Delete user"""
        ...

    @post("/users/{uuid}/actions/revoke", response_class=UpdateUserResponseDto)
    async def revoke_user_subscription(
        self,
        uuid: Annotated[str, Path(description="UUID of the user")],
        body: Optional[Annotated[RevokeUserRequestDto, PydanticBody()]] = None,
    ) -> UpdateUserResponseDto:
        """Revoke User Subscription"""
        ...

    @post("/users/{uuid}/actions/disable", response_class=UpdateUserResponseDto)
    async def disable_user(
        self,
        uuid: Annotated[str, Path(description="UUID of the user")],
    ) -> UpdateUserResponseDto:
        """Disable User"""
        ...

    @post("/users/{uuid}/actions/enable", response_class=UpdateUserResponseDto)
    async def enable_user(
        self,
        uuid: Annotated[str, Path(description="UUID of the user")],
    ) -> UpdateUserResponseDto:
        """Enable User"""
        ...

    @post("/users/{uuid}/actions/reset-traffic", response_class=UpdateUserResponseDto)
    async def reset_user_traffic(
        self,
        uuid: Annotated[str, Path(description="UUID of the user")],
    ) -> UpdateUserResponseDto:
        """Reset User Traffic"""
        ...

    @get("/users/{uuid}", response_class=GetUserByUuidResponseDto)
    async def get_user_by_uuid(
        self,
        uuid: Annotated[str, Path(description="UUID of the user")],
    ) -> GetUserByUuidResponseDto:
        """Get user by UUID"""
        ...

    @get("/users/tags", response_class=GetAllTagsResponseDto)
    async def get_all_tags(
        self,
    ) -> GetAllTagsResponseDto:
        """Get all existing user tags"""
        ...

    @get(
        "/users/{uuid}/accessible-nodes",
        response_class=GetUserAccessibleNodesResponseDto,
    )
    async def get_user_accessible_nodes(
        self,
        uuid: Annotated[str, Path(description="UUID of the user")],
    ) -> GetUserAccessibleNodesResponseDto:
        """Get user accessible nodes"""
        ...

    @get(
        "/users/{uuid}/subscription-request-history", 
        response_class=GetUserSubscriptionRequestHistoryResponseDto
    )
    async def get_user_subscription_request_history(
        self,
        uuid: Annotated[str, Path(description="UUID of the user")],
    ) -> GetUserSubscriptionRequestHistoryResponseDto:
        """Get user subscription request history, recent 24 records"""
        ...

    # ИСПРАВЛЕНО: убран alias, используется short_uuid
    @get("/users/by-short-uuid/{shortUuid}", response_class=GetUserByShortUuidResponseDto)
    async def get_user_by_short_uuid(
        self,
        short_uuid: Annotated[str, Path(description="Short UUID of the user", alias="shortUuid")],
    ) -> GetUserByShortUuidResponseDto:
        """Get user by Short UUID"""
        ...

    @get("/users/by-username/{username}", response_class=GetUserByUsernameResponseDto)
    async def get_user_by_username(
        self,
        username: Annotated[str, Path(description="Username of the user")],
    ) -> GetUserByUsernameResponseDto:
        """Get user by username"""
        ...

    @get("/users/by-id/{id}", response_class=GetUserByIdResponseDto)
    async def get_user_by_id(
        self,
        id: Annotated[str, Path(description="ID of the user")],
    ) -> GetUserByIdResponseDto:
        """Get user by ID"""
        ...

    # ИСПРАВЛЕНО: убран alias, используется telegram_id
    @get(
        "/users/by-telegram-id/{telegramId}",
        response_class=TelegramUserResponseDto,
    )
    async def get_users_by_telegram_id(
        self,
        telegram_id: Annotated[str, Path(description="Telegram ID of the user", alias="telegramId")],
    ) -> TelegramUserResponseDto:
        """Get Users By Telegram ID"""
        ...

    @get("/users/by-email/{email}", response_class=EmailUserResponseDto)
    async def get_users_by_email(
        self,
        email: Annotated[str, Path(description="Email of the user")],
    ) -> EmailUserResponseDto:
        """Get Users By Email"""
        ...

    @get("/users/by-tag/{tag}", response_class=TagUserResponseDto)
    async def get_users_by_tag(
        self,
        tag: Annotated[str, Path(description="Tag of the user")],
    ) -> TagUserResponseDto:
        """Get Users By Tag"""
        ...


from typing import Annotated, Any, List, Optional
from uuid import UUID

from pydantic import BaseModel, Field, StringConstraints

from remnawave.enums import TemplateType


class TemplateResponseDto(BaseModel):
    uuid: UUID
    name: str
    view_position: int = Field(alias="viewPosition")
    template_type: TemplateType = Field(alias="templateType")
    template_json: Any | None = Field(alias="templateJson")
    encoded_template_yaml: str | None = Field(alias="encodedTemplateYaml")


class TemplateInfoDto(BaseModel):
    """Template info without content - used in list responses"""
    uuid: UUID
    name: str
    view_position: int = Field(alias="viewPosition")
    template_type: TemplateType = Field(alias="templateType")
    template_json: Optional[Any] = Field(None, alias="templateJson")
    encoded_template_yaml: Optional[str] = Field(None, alias="encodedTemplateYaml")


class GetTemplateResponseDto(TemplateResponseDto):
    pass

class GetTemplatesData(BaseModel):
    total: int
    templates: List[TemplateInfoDto]

class GetTemplatesResponseDto(GetTemplatesData):
    pass


class CreateSubscriptionTemplateRequestDto(BaseModel):
    name: Annotated[str, StringConstraints(min_length=2, max_length=255, pattern=r"^[A-Za-z0-9_\s-]+$")]
    template_type: TemplateType = Field(serialization_alias="templateType")


class CreateSubscriptionTemplateResponseDto(TemplateResponseDto):
    pass


class UpdateTemplateRequestDto(BaseModel):
    uuid: UUID
    name: Optional[Annotated[str, StringConstraints(min_length=2, max_length=255, pattern=r"^[A-Za-z0-9_\s-]+$")]] = None
    template_json: Optional[dict] = Field(None, serialization_alias="templateJson")
    encoded_template_yaml: Optional[str] = Field(
        None, serialization_alias="encodedTemplateYaml"
    )


class UpdateTemplateResponseDto(TemplateResponseDto):
    pass

class DeleteTemplateData(BaseModel):
    is_deleted: bool = Field(alias="isDeleted")


class DeleteSubscriptionTemplateResponseDto(DeleteTemplateData):
    pass


class ReorderTemplateItem(BaseModel):
    view_position: int = Field(serialization_alias="viewPosition")
    uuid: UUID


class ReorderSubscriptionTemplatesRequestDto(BaseModel):
    items: List[ReorderTemplateItem]


class ReorderSubscriptionTemplatesResponseDto(GetTemplatesData):
    pass


# Legacy aliases for backward compatibility
class UpdateTemplateRequestDtoLegacy(BaseModel):
    template_type: TemplateType = Field(serialization_alias="templateType")
    template_json: Optional[dict] = Field(None, serialization_alias="templateJson")
    encoded_template_yaml: Optional[str] = Field(
        None, serialization_alias="encodedTemplateYaml"
    )


class UpdateTemplateResponseDtoLegacy(TemplateResponseDto):
    pass


import logging
from typing import Optional

import httpx

from remnawave.controllers import (
    APITokensManagementController,
    AuthController,
    BandWidthStatsController,
    ConfigProfilesController,
    HostsBulkActionsController,
    HostsController,
    HWIDUserController,
    InboundsBulkActionsController,
    InboundsController,
    InfraBillingController,
    InternalSquadsController,
    KeygenController,
    NodesController,
    SubscriptionController,
    SubscriptionsController,
    SubscriptionsSettingsController,
    SubscriptionsTemplateController,
    SystemController,
    UsersBulkActionsController,
    UsersController,
    WebhookUtility,
    XrayConfigController,
    SubscriptionRequestHistoryController,
    PasskeysController,
    ExternalSquadsController,
    SnippetsController,
    RemnawaveSettingsController,
    SubscriptionPageConfigController,
    IpControlController,
)
    

class RemnawaveSDK:
    def __init__(
        self,
        client: Optional[httpx.AsyncClient] = None,
        base_url: Optional[str] = None,
        token: Optional[str] = None,
        caddy_token: Optional[str] = None,
        ssl_ignore: Optional[bool] = False,
        custom_headers: Optional[dict] = None,
        cookies: Optional[dict] = None,
    ):
        """
        Remnawave SDK init

        Args:
            client (Optional[httpx.AsyncClient]): - Default client.
            base_url (Optional[str]): - Base url of the Remnawave panel. Defaults to None.
            token (Optional[str]): - Token for authorization.
            caddy_token (Optional[str]): - Token for Caddy Auth (Headers). Defaults to None.
            ssl_ignore (Optional[bool]): - Whether to ignore SSL certificate errors. Defaults to False.
            custom_headers (Optional[dict]): - Custom headers to include in the requests. Defaults to None.
            cookies (Optional[dict]): - Cookies for Reverse Proxy authorization. Defaults to None.
        """
        self._client = client
        self._token = token
        self.base_url = base_url
        self.caddy_token = caddy_token
        self.ssl_ignore = ssl_ignore
        self.custom_headers = custom_headers
        self.cookies = cookies

        self._validate_params()

        if self._client is None:
            self._client = self._prepare_client()

        self.api_tokens_management = APITokensManagementController(self._client)
        self.auth = AuthController(self._client)
        self.bandwidthstats = BandWidthStatsController(self._client)
        self.config_profiles = ConfigProfilesController(self._client)
        self.hosts = HostsController(self._client)
        self.hosts_bulk_actions = HostsBulkActionsController(self._client)
        self.hwid = HWIDUserController(self._client)
        self.inbounds = InboundsController(self._client)
        self.inbounds_bulk_actions = InboundsBulkActionsController(self._client)
        self.infra_billing = InfraBillingController(self._client)
        self.internal_squads = InternalSquadsController(self._client)
        self.keygen = KeygenController(self._client)
        self.nodes = NodesController(self._client)
        self.subscription = SubscriptionController(self._client)
        self.subscriptions = SubscriptionsController(self._client)
        self.subscriptions_settings = SubscriptionsSettingsController(self._client)
        self.subscriptions_template = SubscriptionsTemplateController(self._client)
        self.subscription_request_history = SubscriptionRequestHistoryController(self._client)
        self.system = SystemController(self._client)
        self.users = UsersController(self._client)
        self.users_bulk_actions = UsersBulkActionsController(self._client)
        self.webhook_utility = WebhookUtility()
        self.xray_config = XrayConfigController(self._client)
        self.passkeys = PasskeysController(self._client)
        self.external_squads = ExternalSquadsController(self._client)
        self.snippets = SnippetsController(self._client)
        self.remnawave_settings = RemnawaveSettingsController(self._client)
        self.subscription_page_config = SubscriptionPageConfigController(self._client)
        self.ip_control = IpControlController(self._client)

    def _validate_params(self) -> None:
        if self._client is None:
            if self.base_url is None or self._token is None:
                raise ValueError(
                    "base_url and token must be provided if client is not provided"
                )
        else:
            if self.base_url is not None or self._token is not None:
                logging.warning(
                    "base_url and token will be ignored if client is provided"
                )
            if self.cookies is not None:
                logging.warning(
                    "cookies will be ignored if client is provided"
                )

    def _prepare_client(self) -> httpx.AsyncClient:
        client_kwargs = {
            "base_url": self._prepare_url(),
            "headers": self._prepare_headers(),
            "verify": not self.ssl_ignore,
        }
        if self.cookies is not None:
            client_kwargs["cookies"] = self.cookies
        
        return httpx.AsyncClient(**client_kwargs)

    def _prepare_headers(self) -> dict:
        headers = {}
        if self._token:
            headers["Authorization"] = (
                self._token
                if self._token.startswith("Bearer ")
                else f"Bearer {self._token}"
            )

        # X-Api-Key for Caddy (https://remna.st/security/caddy-with-custom-path#issuing-api-keys)
        if self.caddy_token is not None:
            headers["X-Api-Key"] = self.caddy_token

        if self.custom_headers:
            headers.update(self.custom_headers)

        if "http://" in self.base_url:
            headers["x-forwarded-proto"] = "https"
            headers["x-forwarded-for"] = "127.0.0.1"

        return headers

    def _prepare_url(self) -> str:
        if self.base_url.endswith("/"):
            self.base_url = self.base_url[:-1]

        if not self.base_url.endswith("/api"):
            self.base_url += "/api"

        return self.base_url



from typing import Annotated, Any, List, Optional
from uuid import UUID

from pydantic import BaseModel, ConfigDict, Field, StringConstraints


class SubscriptionPageConfigDto(BaseModel):
    """Subscription page config data model"""
    model_config = ConfigDict(populate_by_name=True)
    
    uuid: UUID
    view_position: int = Field(alias="viewPosition")
    name: str
    config: Any | None


class GetSubscriptionPageConfigsData(BaseModel):
    """Data for getting all subscription page configs"""
    total: int
    configs: List[SubscriptionPageConfigDto]


class GetSubscriptionPageConfigsResponseDto(GetSubscriptionPageConfigsData):
    """Response with all subscription page configs"""
    pass


class GetSubscriptionPageConfigResponseDto(BaseModel):
    """Response with single subscription page config"""
    model_config = ConfigDict(populate_by_name=True)
    
    uuid: UUID
    view_position: int = Field(alias="viewPosition")
    name: str
    config: Any


class CreateSubscriptionPageConfigRequestDto(BaseModel):
    """Request to create subscription page config"""
    name: Annotated[
        str,
        StringConstraints(min_length=2, max_length=30, pattern=r"^[A-Za-z0-9_\s-]+$")
    ]


class CreateSubscriptionPageConfigResponseDto(SubscriptionPageConfigDto):
    """Response after creating subscription page config"""
    pass


class UpdateSubscriptionPageConfigRequestDto(BaseModel):
    """Request to update subscription page config"""
    model_config = ConfigDict(populate_by_name=True)
    
    uuid: UUID
    name: Optional[Annotated[
        str,
        StringConstraints(min_length=2, max_length=30, pattern=r"^[A-Za-z0-9_\s-]+$")
    ]] = None
    config: Optional[Any] = None


class UpdateSubscriptionPageConfigResponseDto(SubscriptionPageConfigDto):
    """Response after updating subscription page config"""
    pass


class DeleteSubscriptionPageConfigData(BaseModel):
    """Data for delete response"""
    model_config = ConfigDict(populate_by_name=True)
    
    is_deleted: bool = Field(alias="isDeleted")


class DeleteSubscriptionPageConfigResponseDto(DeleteSubscriptionPageConfigData):
    """Response after deleting subscription page config"""
    pass


class ReorderSubscriptionPageConfigItem(BaseModel):
    """Item for reordering subscription page configs"""
    model_config = ConfigDict(populate_by_name=True)
    
    view_position: int = Field(alias="viewPosition")
    uuid: UUID


class ReorderSubscriptionPageConfigsRequestDto(BaseModel):
    """Request to reorder subscription page configs"""
    items: List[ReorderSubscriptionPageConfigItem]


class ReorderSubscriptionPageConfigsResponseDto(GetSubscriptionPageConfigsData):
    """Response after reordering subscription page configs"""
    pass


class CloneSubscriptionPageConfigRequestDto(BaseModel):
    """Request to clone subscription page config"""
    model_config = ConfigDict(populate_by_name=True)
    
    clone_from_uuid: UUID = Field(alias="cloneFromUuid")


class CloneSubscriptionPageConfigResponseDto(SubscriptionPageConfigDto):
    """Response after cloning subscription page config"""
    pass


class GetSubpageConfigByShortUuidRequestBodyDto(BaseModel):
    """Request body for getting subpage config by short UUID"""
    model_config = ConfigDict(populate_by_name=True)
    
    request_headers: dict[str, str] = Field(default_factory=dict, serialization_alias="requestHeaders")


class SubpageConfigData(BaseModel):
    """Data inside GetSubpageConfigByShortUuidResponseDto"""
    model_config = ConfigDict(populate_by_name=True)
    
    subpage_config_uuid: UUID | None = Field(alias="subpageConfigUuid")
    webpage_allowed: bool = Field(alias="webpageAllowed")


class GetSubpageConfigByShortUuidResponseDto(BaseModel):
    """Response for getting subpage config by short UUID"""
    model_config = ConfigDict(populate_by_name=True)
    
    response: SubpageConfigData


from datetime import datetime
from typing import List, Optional
from uuid import UUID

from pydantic import BaseModel, Field


class CreateUserHwidDeviceRequestDto(BaseModel):
    hwid: str
    user_uuid: UUID = Field(serialization_alias="userUuid")
    platform: Optional[str] = None
    os_version: Optional[str] = Field(None, serialization_alias="osVersion")
    device_model: Optional[str] = Field(None, serialization_alias="deviceModel")
    user_agent: Optional[str] = Field(None, serialization_alias="userAgent")


class DeleteUserHwidDeviceRequestDto(BaseModel):
    user_uuid: UUID = Field(serialization_alias="userUuid")
    hwid: str


class HwidDeviceDto(BaseModel):
    hwid: str
    user_uuid: UUID = Field(alias="userUuid")
    platform: Optional[str] = None
    os_version: Optional[str] = Field(None, alias="osVersion")
    device_model: Optional[str] = Field(None, alias="deviceModel")
    user_agent: Optional[str] = Field(None, alias="userAgent")
    created_at: datetime = Field(alias="createdAt")
    updated_at: datetime = Field(alias="updatedAt")


class HwidDevicesData(BaseModel):
    total: int
    devices: List[HwidDeviceDto]


class CreateUserHwidDeviceResponseDto(BaseModel):
    total: int
    devices: List[HwidDeviceDto]


class DeleteUserHwidDeviceResponseDto(BaseModel):
    total: int
    devices: List[HwidDeviceDto]


class GetUserHwidDevicesResponseDto(BaseModel):
    total: int
    devices: List[HwidDeviceDto]

class PlatformStatItem(BaseModel):
    platform: str
    count: float


class AppStatItem(BaseModel):
    app: str
    count: float


class HwidStats(BaseModel):
    total_unique_devices: float = Field(alias="totalUniqueDevices")
    total_hwid_devices: float = Field(alias="totalHwidDevices")
    average_hwid_devices_per_user: float = Field(alias="averageHwidDevicesPerUser")


class HwidStatisticsData(BaseModel):
    by_platform: List[PlatformStatItem] = Field(alias="byPlatform")
    by_app: List[AppStatItem] = Field(alias="byApp")
    stats: HwidStats


class GetHwidStatisticsResponseDto(HwidStatisticsData):
    pass

class DeleteUserAllHwidDeviceRequestDto(BaseModel):
    user_uuid: UUID = Field(serialization_alias="userUuid")
    
class TopUserByHwidDevicesDto(BaseModel):
    """Top user by HWID devices"""
    user_uuid: UUID = Field(alias="userUuid")
    id: int
    username: str
    devices_count: int = Field(alias="devicesCount")


class TopUsersByHwidDevicesData(BaseModel):
    """Top users by HWID devices data"""
    users: list[TopUserByHwidDevicesDto]
    total: int


class GetTopUsersByHwidDevicesResponseDto(TopUsersByHwidDevicesData):
    """Response for get top users by HWID devices"""
    pass

# Legacy aliases for backward compatibility
CreateHWIDUser = CreateUserHwidDeviceRequestDto
HWIDUserResponseDto = HwidDeviceDto
HWIDUserResponseDtoList = HwidDevicesData
HWIDDeleteRequest = DeleteUserHwidDeviceRequestDto


from typing import List
from uuid import UUID

from pydantic import BaseModel, Field

from remnawave.models import HostResponseDto


class SetInboundToManyHostsRequestDto(BaseModel):
    uuids: List[UUID]
    config_profile_uuid: UUID = Field(serialization_alias="configProfileUuid")
    config_profile_inbound_uuid: UUID = Field(
        serialization_alias="configProfileInboundUuid"
    )

class SetPortToManyHostsRequestDto(BaseModel):
    uuids: List[UUID]
    port: int = Field(ge=1, le=65535)

class BulkDeleteHostsResponseDto(List[HostResponseDto]):
    pass


class BulkDisableHostsResponseDto(List[HostResponseDto]):
    pass


class BulkEnableHostsResponseDto(List[HostResponseDto]):
    pass


class SetInboundToManyHostsResponseDto(List[HostResponseDto]):
    pass

class SetPortToManyHostsResponseDto(List[HostResponseDto]):
    pass



from typing import Annotated

from rapid_api_client.annotations import JsonBody

from remnawave.models import GetConfigResponseDto, UpdateConfigResponseDto
from remnawave.rapid import BaseController, get, put


class XrayConfigController(BaseController):
    @get("/xray", response_class=GetConfigResponseDto)
    async def get_config(
        self,
    ) -> GetConfigResponseDto:
        """Get Xray Config"""
        ...

    @put("/xray", response_class=UpdateConfigResponseDto)
    async def update_config(
        self,
        body: Annotated[dict, JsonBody()],
    ) -> UpdateConfigResponseDto:
        """Update Xray Config"""
        ...



from typing import Annotated

from rapid_api_client import Path, Query

from remnawave.models import (
    GetNodeUserUsageByRangeResponseDto,
    GetNodesUsageByRangeResponseDto,
)
from remnawave.rapid import BaseController, get



from .api_tokens_management import (
    CreateApiTokenRequestDto,
    CreateApiTokenResponseDto,
    DeleteApiTokenResponseDto,
    FindAllApiTokensResponseDto,
)
from .auth import (
    GetStatusResponseDto,
    LoginRequestDto,
    LoginResponseDto,
    RegisterRequestDto,
    RegisterResponseDto,
    StatusResponseDto,  # Legacy alias
    TelegramCallbackRequestDto,
    TelegramCallbackResponseDto,
    LoginTelegramRequestDto,  # Legacy alias
    OAuth2AuthorizeRequestDto,
    OAuth2AuthorizeResponseDto,
    OAuth2CallbackRequestDto,
    OAuth2CallbackResponseDto,
    VerifyPasskeyAuthenticationRequestDto,
    VerifyPasskeyAuthenticationResponseDto,
    GetPasskeyAuthenticationOptionsResponseDto,
)
from .bandwidthstats import (
    GetNodeUserUsageByRangeResponseDto,
    GetNodesRealtimeUsageResponseDto,
    GetNodesUsageByRangeResponseDto,
    GetUserUsageByRangeResponseDto,
    NodeRealtimeUsageResponseDto,
    NodeUsageResponseDto,
    NodesRealtimeUsageResponseDto,  # Legacy alias
    NodesUsageResponseDto,  # Legacy alias
    GetLegacyStatsUserUsageResponseDto,
    GetLegacyStatsNodesUsersUsageResponseDto,
    GetStatsNodesRealtimeUsageResponseDto,
    GetStatsNodesUsageResponseDto,
    GetStatsNodeUsersUsageResponseDto,
    GetStatsUserUsageResponseDto,

    # Data Models
    LegacyUserUsageItem,
    LegacyNodeUserUsageItem,
    NodeRealtimeUsageItem,
    TopNodeItem,
    TopUserItem,
    NodeSeriesItem,
    StatsNodesUsageData,
    StatsNodeUsersUsageData,
    StatsUserUsageData,
)
from .config_profiles import (
    ConfigProfileDto,
    CreateConfigProfileRequestDto,
    CreateConfigProfileResponseDto,
    DeleteConfigProfileResponseDto,
    GetAllConfigProfilesResponseDto,
    GetAllConfigProfilesResponsePaginated,
    GetAllInboundsResponseDto,
    GetConfigProfileByUuidResponseDto,
    GetInboundsByProfileUuidResponseDto,
    InboundDto,
    NodesProfileDto,
    ReorderConfigProfileItem,
    ReorderConfigProfilesRequestDto,
    ReorderConfigProfilesResponseDto,
    UpdateConfigProfileRequestDto,
    UpdateConfigProfileResponseDto,
)
from .hosts import (
    CreateHostInboundData,
    CreateHostRequestDto,
    CreateHostResponseDto,
    DeleteHostResponseDto,
    GetAllHostsResponseDto,
    GetOneHostResponseDto,
    HostInboundData,
    HostResponseDto,
    HostsResponseDto,  # Legacy alias
    ReorderHostItem,
    ReorderHostRequestDto,
    ReorderHostResponseDto,
    UpdateHostRequestDto,
    UpdateHostResponseDto,
    GetAllHostTagsResponseDto,
)
from .hosts_bulk_actions import (
    BulkDeleteHostsResponseDto,
    BulkDisableHostsResponseDto,
    BulkEnableHostsResponseDto,
    SetInboundToManyHostsRequestDto,
    SetInboundToManyHostsResponseDto,
    SetPortToManyHostsResponseDto,
    SetPortToManyHostsRequestDto
)
from .hwid import (
    CreateHWIDUser,  # Legacy alias
    CreateUserHwidDeviceRequestDto,
    CreateUserHwidDeviceResponseDto,
    DeleteUserHwidDeviceRequestDto,
    DeleteUserHwidDeviceResponseDto,
    GetUserHwidDevicesResponseDto,
    HWIDDeleteRequest,  # Legacy alias
    HWIDUserResponseDto,  # Legacy alias
    HWIDUserResponseDtoList,  # Legacy alias
    GetHwidStatisticsResponseDto,
    DeleteUserAllHwidDeviceRequestDto,
    GetTopUsersByHwidDevicesResponseDto,
    TopUserByHwidDevicesDto,
    TopUsersByHwidDevicesData,
)
from .inbounds import (
    AllInboundsData,
    FullInboundResponseDto,
    FullInboundStatistic,
    FullInboundsResponseDto,
    GetAllInboundsResponseDto,
    GetFullInboundsResponseDto,
    GetInboundsByProfileUuidResponseDto,
    GetInboundsResponseDto,
    InboundResponseDto,
    InboundsByProfileData,
    InboundsResponseDto,  # Legacy alias
)
from .inbounds_bulk_actions import (
    AddInboundToNodesResponseDto,
    AddInboundToUsersResponseDto,
    RemoveInboundFromNodesResponseDto,
    RemoveInboundFromUsersResponseDto,
)
from .infra_billing import (
    CreateInfraBillingHistoryRecordRequestDto,
    CreateInfraBillingHistoryRecordResponseDto, 
    CreateInfraBillingNodeRequestDto,
    CreateInfraBillingNodeResponseDto,
    CreateInfraProviderRequestDto,
    CreateInfraProviderResponseDto,
    DeleteInfraBillingHistoryRecordByUuidResponseDto,  
    DeleteInfraBillingNodeByUuidResponseDto,  # ПЕРЕИМЕНОВАНА (было DeleteInfraBillingNodeResponseDto)
    DeleteInfraProviderByUuidResponseDto,  # ПЕРЕИМЕНОВАНА (было DeleteInfraProviderResponseDto)
    GetInfraBillingHistoryRecordsResponseDto,  # ПЕРЕИМЕНОВАНА (было GetAllInfraBillingHistoryResponseDto)
    GetInfraBillingNodesResponseDto,  # ПЕРЕИМЕНОВАНА (было GetAllInfraBillingNodesResponseDto)
    GetInfraProvidersResponseDto,  # ПЕРЕИМЕНОВАНА (было GetAllInfraProvidersResponseDto)
    GetInfraBillingHistoryByUuidResponseDto,
    GetInfraBillingNodeByUuidResponseDto,
    GetInfraProviderByUuidResponseDto,
    InfraBillingHistoryDto,
    InfraBillingNodeDto,
    InfraProviderDto,
    NodeDto,
    UpdateInfraBillingNodeRequestDto,
    UpdateInfraBillingNodeResponseDto,
    UpdateInfraProviderRequestDto,
    UpdateInfraProviderResponseDto,
    DeleteInfraBillingNodeResponseDto,  # LEGACY
    DeleteInfraProviderResponseDto,  # LEGACY
    GetAllInfraBillingHistoryResponseDto,  # LEGACY
    GetAllInfraBillingNodesResponseDto,  # LEGACY
    GetAllInfraProvidersResponseDto,  # LEGACY
)
from .internal_squads import (
    AddUsersToInternalSquadRequestDto,
    AddUsersToInternalSquadResponseDto,
    CreateInternalSquadRequestDto,
    CreateInternalSquadResponseDto,
    DeleteInternalSquadResponseDto,
    DeleteUsersFromInternalSquadRequestDto,
    DeleteUsersFromInternalSquadResponseDto,
    GetAllInternalSquadsResponseDto,
    GetInternalSquadByUuidResponseDto,
    InternalSquadDto,
    ReorderInternalSquadItem,
    ReorderInternalSquadsRequestDto,
    ReorderInternalSquadsResponseDto,
    UpdateInternalSquadRequestDto,
    UpdateInternalSquadResponseDto,
    GetInternalSquadAccessibleNodesResponseDto,
)
from .keygen import GetPubKeyResponseDto, PubKeyResponseDto  # Legacy alias
from .nodes import (
    CreateNodeRequestDto,
    CreateNodeResponseDto,
    DeleteNodeResponseDto,
    DisableNodeResponseDto,
    EnableNodeResponseDto,
    ExcludedInbounds,
    GetAllNodesResponseDto,
    GetAllNodesTagsResponseDto,
    GetOneNodeResponseDto,
    NodeConfigProfileDto,
    NodeConfigProfileRequestDto,
    NodeResponseDto,
    NodesResponseDto,  # Legacy alias
    ReorderNodeRequestDto,
    ReorderNodeResponseDto,
    RestartAllNodesResponseDto,
    RestartNodeResponseDto,
    UpdateNodeRequestDto,
    UpdateNodeResponseDto,
    RestartAllNodesRequestDto, # Legacy alias,
    RestartAllNodesRequestBodyDto,
    ResetNodeTrafficRequestDto,
    ResetNodeTrafficResponseDto,
    ProfileModificationRequestDto,
    ProfileModificationResponseDto,
    NodeBulkActionType,
    NodesBulkActionsRequestDto,
    NodesBulkActionsResponseDto,
)
from .nodes_usage_history import (
    GetNodeUserUsageByRangeResponseDto,
    GetNodesUsageByRangeResponseDto,
    GetUserAccessibleNodesResponseDto,
    NodeInfoDto,
    NodeUsageDto,
    UserUsageDto,
)
from .subscription import (
    GetAllSubscriptionsResponseDto,
    GetSubscriptionByUsernameResponseDto,
    GetSubscriptionInfoResponseDto,
    SubscriptionInfoResponseDto,  # Legacy alias
    UserSubscription,
    GetRawSubscriptionByShortUuidResponseDto,
    RawSettings,
    GetSubscriptionByShortUUIDResponseDto,
    GetSubscriptionByUUIDResponseDto,
)
from .subscriptions_settings import (
    GetSubscriptionSettingsResponseDto,
    ResponseModificationHeader,
    ResponseModifications,
    ResponseRule,
    ResponseRuleCondition,
    ResponseRules,
    SubscriptionSettingsResponseDto,
    SubscriptionType,
    UpdateSubscriptionSettingsRequestDto,
    UpdateSubscriptionSettingsResponseDto,
    CustomRemarksDto,
    HwidSettingsDto,
    # Backward compatibility aliases
    CustomRemarks,
    HwidSettings,
)
from .subscriptions_template import (
    CreateSubscriptionTemplateRequestDto,
    CreateSubscriptionTemplateResponseDto,
    DeleteSubscriptionTemplateResponseDto,
    GetTemplatesResponseDto,
    TemplateInfoDto,
    GetTemplateResponseDto,
    ReorderTemplateItem,
    ReorderSubscriptionTemplatesRequestDto,
    ReorderSubscriptionTemplatesResponseDto,
    TemplateResponseDto,
    UpdateTemplateRequestDto,
    UpdateTemplateResponseDto,
)
from .system import (
    BandwidthStatistic,
    BandwidthStatisticResponseDto,
    CPUStatistic,
    GetBandwidthStatsResponseDto,
    GetNodesStatisticsResponseDto,
    GetRemnawaveHealthResponseDto,
    GetStatsResponseDto,
    MemoryStatistic,
    NodeStatistic,
    NodesStatisticResponseDto,
    OnlineStatistic,
    StatisticResponseDto,
    StatusCounts,
    UsersStatistic,
    GetNodesMetricsResponseDto,
    GetX25519KeyPairResponseDto, 
    X25519KeyPair,
    DebugSrrMatcherRequestDto,
    DebugSrrMatcherResponseDto,
    EncryptHappCryptoLinkRequestDto,
    EncryptHappCryptoLinkResponseDto,
    GetMetadataResponseDto
)
from .users import (
    # Request DTOs
    CreateUserRequestDto,
    UpdateUserRequestDto,
    RevokeUserRequestDto,
    
    # Response DTOs - Single User
    CreateUserResponseDto,
    UpdateUserResponseDto,
    GetUserByUuidResponseDto,
    GetUserByShortUuidResponseDto,
    GetUserByUsernameResponseDto,
    GetUserByIdResponseDto,
    DisableUserResponseDto,
    EnableUserResponseDto,
    ResetUserTrafficResponseDto,
    RevokeUserSubscriptionResponseDto,
    ActivateAllInboundsResponseDto,
    
    # Response DTOs - Collections
    GetAllUsersResponseDto,
    GetAllTagsResponseDto,
    GetUserSubscriptionRequestHistoryResponseDto,
    
    # Response DTOs - Arrays (RootModel)
    TelegramUserResponseDto,
    EmailUserResponseDto,
    TagUserResponseDto,
    
    # Other Response DTOs
    DeleteUserResponseDto,
    
    # Base DTOs
    UserResponseDto,
    UsersResponseDto,
    TagsResponseDto,
    SubscriptionRequestsResponseData,
    
    # Data DTOs
    UserTrafficDto,
    ActiveInternalSquadDto,
    SubscriptionRequestRecord,
    HappCrypto,
    UserActiveInboundsDto,
    UserLastConnectedNodeDto,

    # Alias
    GetSubscriptionRequestsResponseDto,
)

from .users_bulk_actions import (
    BulkAllExtendExpirationDateRequestDto, 
    BulkAllExtendExpirationDateResponseDto, 
    BulkAllResetTrafficUsersResponseDto,
    BulkAllUpdateUsersRequestDto,
    BulkAllUpdateUsersResponseDto,
    BulkDeleteUsersByStatusRequestDto, 
    BulkDeleteUsersByStatusResponseDto, 
    BulkDeleteUsersRequestDto, 
    BulkDeleteUsersResponseDto, 
    BulkEventResponseData, 
    BulkExtendExpirationDateRequestDto, 
    BulkExtendExpirationDateResponseDto, 
    BulkResetTrafficUsersRequestDto, 
    BulkResetTrafficUsersResponseDto, 
    BulkResponseData, 
    BulkResponseDto,
    BulkRevokeUsersSubscriptionRequestDto, 
    BulkRevokeUsersSubscriptionResponseDto, 
    BulkUpdateUsersInternalSquadsRequestDto,
    BulkUpdateUsersRequestDto, 
    BulkUpdateUsersResponseDto, 
    BulkUpdateUsersSquadsRequestDto, 
    BulkUpdateUsersSquadsResponseDto, 
    TagStr, 
    UpdateUserFields,
)
from .users_stats import UserUsageByRange, UserUsageByRangeResponseDto
from .xray_config import (
    ConfigResponseDto,  # Legacy alias
    GetConfigResponseDto,
    UpdateConfigRequestDto,
    UpdateConfigResponseDto,
)
from .subscription_request_history import (
    GetAllSubscriptionRequestHistoryResponseDto,
    GetSubscriptionRequestHistoryStatsResponseDto,
    SubscriptionRequestHistoryRecord,
    SubscriptionRequestHistoryData,
    AppStatItem,
    HourlyRequestStat,
    SubscriptionRequestHistoryStatsData
)
from .webhook import (
    UserEventDto, 
    UserHwidDeviceEventDto,
    HwidUserDeviceDto,
    LastConnectedNodeDto,
    InternalSquadDto,
    BaseUserDto,
    UserDto,
    NodeDto,
    WebhookNodeConfigProfileDto,
    ConfigProfileInboundDto,
    InfraProviderDto,
    LoginAttemptDto,
    ServiceEventDto,
    NodeEventDto,
    CustomErrorEventDto,
    CrmEventDto,
    WebhookPayloadDto,
    UserTrafficDto
)
from .passkeys import (
    DeletePasskeyRequestDto,
    DeletePasskeyResponseDto,
    GetAllPasskeysResponseDto,
    GetPasskeyRegistrationOptionsResponseDto,
    PasskeyDto,
    UpdatePasskeyRequestDto,
    UpdatePasskeyResponseDto,
    VerifyPasskeyRegistrationRequestDto,
    VerifyPasskeyRegistrationResponseDto,
)
from .external_squads import (
    AddUsersToExternalSquadResponseDto,
    CreateExternalSquadRequestDto,
    CreateExternalSquadResponseDto,
    DeleteExternalSquadResponseDto,
    ExternalSquadDto,
    ExternalSquadInfoDto,
    ExternalSquadSubscriptionSettingsDto,
    ExternalSquadTemplateDto,
    GetExternalSquadByUuidResponseDto,
    GetExternalSquadsResponseDto,
    RemoveUsersFromExternalSquadResponseDto,
    ReorderExternalSquadItem,
    ReorderExternalSquadsRequestDto,
    ReorderExternalSquadsResponseDto,
    TemplateType,
    UpdateExternalSquadRequestDto,
    UpdateExternalSquadResponseDto,
)
from .snippets import (
    CreateSnippetRequestDto,
    CreateSnippetResponseDto,
    DeleteSnippetRequestDto,
    DeleteSnippetResponseDto,
    GetSnippetsResponseDto,
    SnippetItem,
    SnippetsData,
    UpdateSnippetRequestDto,
    UpdateSnippetResponseDto,
)
from .remnawave_settings import (
    BrandingSettings,
    GetRemnawaveSettingsResponseDto,
    GitHubOAuth2Settings,
    GenericOAuth2Settings,
    KeycloakOAuth2Settings,
    OAuth2Settings,
    PasskeySettings,
    PasswordSettings,
    PocketIdOAuth2Settings,
    RemnawaveSettingsData,
    TelegramAuthSettings,
    UpdateRemnawaveSettingsRequestDto,
    UpdateRemnawaveSettingsResponseDto,
    YandexOAuth2Settings,
)
from .subscription_page import (
    CloneSubscriptionPageConfigRequestDto,
    CloneSubscriptionPageConfigResponseDto,
    CreateSubscriptionPageConfigRequestDto,
    CreateSubscriptionPageConfigResponseDto,
    DeleteSubscriptionPageConfigResponseDto,
    GetSubscriptionPageConfigResponseDto,
    GetSubscriptionPageConfigsResponseDto,
    GetSubpageConfigByShortUuidRequestBodyDto,
    GetSubpageConfigByShortUuidResponseDto,
    SubpageConfigData,
    ReorderSubscriptionPageConfigItem,
    ReorderSubscriptionPageConfigsRequestDto,
    ReorderSubscriptionPageConfigsResponseDto,
    SubscriptionPageConfigDto,
    UpdateSubscriptionPageConfigRequestDto,
    UpdateSubscriptionPageConfigResponseDto,
)
from .ip_control import (
    # Request DTOs
    DropConnectionsRequestDto,
    DropByUserUuids,
    DropByIpAddresses,
    TargetAllNodes,
    TargetSpecificNodes,
    # Response DTOs
    FetchIpsResponseDto,
    FetchIpsResultResponseDto,
    DropConnectionsResponseDto,
    # Data models
    FetchIpsJobData,
    FetchIpsProgressData,
    FetchIpsNodeResult,
    FetchIpsResult,
    FetchIpsResultData,
    DropConnectionsResponseData,
)

__all__ = [
    # Auth models
    "GetStatusResponseDto",
    "LoginRequestDto",
    "LoginResponseDto",
    "RegisterRequestDto",
    "RegisterResponseDto",
    "StatusResponseDto",  # Legacy alias
    "TelegramCallbackRequestDto",
    "TelegramCallbackResponseDto",
    "LoginTelegramRequestDto",  # Legacy alias
    "OAuth2AuthorizeRequestDto",
    "OAuth2AuthorizeResponseDto",
    "OAuth2CallbackRequestDto",
    "OAuth2CallbackResponseDto",
    "VerifyPasskeyAuthenticationRequestDto",
    "VerifyPasskeyAuthenticationResponseDto",
    "GetPasskeyAuthenticationOptionsResponseDto",
    # Nodes models
    "CreateNodeRequestDto",
    "CreateNodeResponseDto",
    "DeleteNodeResponseDto",
    "DisableNodeResponseDto",
    "EnableNodeResponseDto",
    "ExcludedInbounds",
    "GetAllNodesResponseDto",
    "GetAllNodesTagsResponseDto",
    "GetOneNodeResponseDto",
    "NodeResponseDto",
    "NodesResponseDto",  # Legacy alias
    "ReorderNodeRequestDto",
    "ReorderNodeResponseDto",
    "RestartAllNodesResponseDto",
    "RestartNodeResponseDto",
    "UpdateNodeRequestDto",
    "UpdateNodeResponseDto",
    "NodeConfigProfileDto",
    "NodeConfigProfileRequestDto",
    "RestartAllNodesRequestDto",  # Legacy alias
    "RestartAllNodesRequestBodyDto",
    "ResetNodeTrafficRequestDto",
    "ResetNodeTrafficResponseDto",
    "ProfileModificationRequestDto",
    "ProfileModificationResponseDto",
    "NodeBulkActionType",
    "NodesBulkActionsRequestDto",
    "NodesBulkActionsResponseDto",
    # Hosts models
    "CreateHostRequestDto",
    "CreateHostResponseDto",
    "DeleteHostResponseDto",
    "GetAllHostsResponseDto",
    "GetOneHostResponseDto",
    "HostResponseDto",
    "HostsResponseDto",  # Legacy alias
    "ReorderHostRequestDto",
    "ReorderHostResponseDto",
    "UpdateHostRequestDto",
    "UpdateHostResponseDto",
    "GetAllHostTagsResponseDto",
    "CreateHostInboundData",
    "HostInboundData",
    "ReorderHostItem",
    # Inbounds models
    "AllInboundsData",
    "FullInboundResponseDto",
    "FullInboundStatistic",
    "FullInboundsResponseDto",
    "GetFullInboundsResponseDto",
    "GetInboundsResponseDto",
    "InboundResponseDto",
    "InboundsResponseDto",  # Legacy alias
    "GetInternalSquadAccessibleNodesResponseDto",
    "InboundsByProfileData",
    # Keygen models
    "GetPubKeyResponseDto",
    "PubKeyResponseDto",  # Legacy alias
    # Subscription models
    "GetAllSubscriptionsResponseDto",
    "GetSubscriptionByUsernameResponseDto",
    "GetSubscriptionByShortUUIDResponseDto",
    "GetSubscriptionByUUIDResponseDto",
    "GetSubscriptionInfoResponseDto",
    "SubscriptionInfoResponseDto",  # Legacy alias
    "UserSubscription",
    "GetRawSubscriptionByShortUuidResponseDto",
    "RawSettings",
    # Subscription settings models
    "GetSubscriptionSettingsResponseDto",
    "SubscriptionSettingsResponseDto",
    "UpdateSubscriptionSettingsRequestDto",
    "SubscriptionType",
    "CustomRemarksDto",
    "HwidSettingsDto",
    # Backward compatibility aliases
    "CustomRemarks",
    "HwidSettings",
    "UpdateSubscriptionSettingsResponseDto",
    "ResponseModificationHeader",
    "ResponseModifications",
    "ResponseRule",
    "ResponseRuleCondition",
    "ResponseRules",
    # Subscription template models
    "GetTemplateResponseDto",
    "TemplateResponseDto",
    "UpdateTemplateRequestDto",
    "UpdateTemplateResponseDto",
    "CreateSubscriptionTemplateRequestDto",
    "CreateSubscriptionTemplateResponseDto",
    "DeleteSubscriptionTemplateResponseDto",
    "GetTemplatesResponseDto",
    "ReorderTemplateItem",
    "ReorderSubscriptionTemplatesRequestDto",
    "ReorderSubscriptionTemplatesResponseDto",
    "TemplateInfoDto",
    # System models
    "BandwidthStatistic",
    "BandwidthStatisticResponseDto",
    "CPUStatistic",
    "GetBandwidthStatsResponseDto",
    "GetNodesStatisticsResponseDto",
    "GetRemnawaveHealthResponseDto",
    "GetStatsResponseDto",
    "MemoryStatistic",
    "NodeStatistic",
    "NodesStatisticResponseDto",
    "OnlineStatistic",
    "StatisticResponseDto",
    "StatusCounts",
    "UsersStatistic",
    "GetNodesMetricsResponseDto",
    "GetX25519KeyPairResponseDto",
    "X25519KeyPair",
    "DebugSrrMatcherRequestDto",
    "DebugSrrMatcherResponseDto",
    "EncryptHappCryptoLinkRequestDto",
    "EncryptHappCryptoLinkResponseDto",
    "GetMetadataResponseDto"
    # XRay config models
    "ConfigResponseDto",  # Legacy alias
    "GetConfigResponseDto",
    "UpdateConfigRequestDto",
    "UpdateConfigResponseDto",
    # HWID models
    "CreateHWIDUser",  # Legacy alias
    "CreateUserHwidDeviceRequestDto",
    "CreateUserHwidDeviceResponseDto",
    "DeleteUserHwidDeviceRequestDto",
    "DeleteUserHwidDeviceResponseDto",
    "GetUserHwidDevicesResponseDto",
    "HWIDDeleteRequest",  # Legacy alias
    "HWIDUserResponseDto",  # Legacy alias
    "HWIDUserResponseDtoList",  # Legacy alias
    "GetHwidStatisticsResponseDto",
    "DeleteUserAllHwidDeviceRequestDto",
    "GetTopUsersByHwidDevicesResponseDto",
    "TopUserByHwidDevicesDto",
    "TopUsersByHwidDevicesData",
    # Bandwidth stats models
    "GetNodeUserUsageByRangeResponseDto",
    "GetNodesRealtimeUsageResponseDto",
    "GetNodesUsageByRangeResponseDto",
    "GetUserUsageByRangeResponseDto",
    "NodeRealtimeUsageResponseDto",
    "NodeUsageResponseDto",
    "NodesRealtimeUsageResponseDto",  # Legacy alias
    "NodesUsageResponseDto",  # Legacy alias
    "GetLegacyStatsUserUsageResponseDto",
    "GetLegacyStatsNodesUsersUsageResponseDto",
    "GetStatsNodesRealtimeUsageResponseDto",
    "GetStatsNodesUsageResponseDto",
    "GetStatsNodeUsersUsageResponseDto",
    "GetStatsUserUsageResponseDto",
    "LegacyUserUsageItem",
    "LegacyNodeUserUsageItem",
    "NodeRealtimeUsageItem",
    "TopNodeItem",
    "TopUserItem",
    "NodeSeriesItem",
    "StatsNodesUsageData",
    "StatsNodeUsersUsageData",
    "StatsUserUsageData",
    # API Tokens models
    "CreateApiTokenRequestDto",
    "CreateApiTokenResponseDto",
    "DeleteApiTokenResponseDto",
    "FindAllApiTokensResponseDto",
    # Inbound bulk actions models
    "AddInboundToNodesResponseDto",
    "AddInboundToUsersResponseDto",
    "RemoveInboundFromNodesResponseDto",
    "RemoveInboundFromUsersResponseDto",
    # Host bulk actions models
    "BulkDeleteHostsResponseDto",
    "BulkDisableHostsResponseDto",
    "BulkEnableHostsResponseDto",
    "SetInboundToManyHostsRequestDto",
    "SetInboundToManyHostsResponseDto",
    "SetPortToManyHostsResponseDto",
    "SetPortToManyHostsRequestDto",
    # Users models
    "CreateUserRequestDto",
    "UpdateUserRequestDto",
    "RevokeUserRequestDto",
    "CreateUserResponseDto",
    "UpdateUserResponseDto",
    "GetUserByUuidResponseDto",
    "GetUserByShortUuidResponseDto",
    "GetUserByUsernameResponseDto",
    "GetUserByIdResponseDto",
    "DisableUserResponseDto",
    "EnableUserResponseDto",
    "ResetUserTrafficResponseDto",
    "RevokeUserSubscriptionResponseDto",
    "ActivateAllInboundsResponseDto",
    "GetAllUsersResponseDto",
    "GetAllTagsResponseDto",
    "GetUserSubscriptionRequestHistoryResponseDto",
    "TelegramUserResponseDto",
    "EmailUserResponseDto",
    "TagUserResponseDto",
    "DeleteUserResponseDto",
    "UserResponseDto",
    "UsersResponseDto",
    "TagsResponseDto",
    "SubscriptionRequestsResponseData",
    "UserTrafficDto",
    "ActiveInternalSquadDto",
    "SubscriptionRequestRecord",
    "HappCrypto",
    "UserActiveInboundsDto",
    "UserLastConnectedNodeDto",
    "GetSubscriptionRequestsResponseDto",
    # Users bulk actions models
    "BulkAllExtendExpirationDateRequestDto",
    "BulkAllExtendExpirationDateResponseDto",
    "BulkAllResetTrafficUsersResponseDto",
    "BulkAllUpdateUsersRequestDto",
    "BulkAllUpdateUsersResponseDto",
    "BulkDeleteUsersByStatusRequestDto",
    "BulkDeleteUsersByStatusResponseDto",
    "BulkDeleteUsersRequestDto",
    "BulkDeleteUsersResponseDto",
    "BulkEventResponseData",
    "BulkExtendExpirationDateRequestDto",
    "BulkExtendExpirationDateResponseDto",
    "BulkResetTrafficUsersRequestDto",
    "BulkResetTrafficUsersResponseDto",
    "BulkResponseData",
    "BulkResponseDto",
    "BulkRevokeUsersSubscriptionRequestDto",
    "BulkRevokeUsersSubscriptionResponseDto",
    "BulkUpdateUsersInternalSquadsRequestDto",
    "BulkUpdateUsersRequestDto",
    "BulkUpdateUsersResponseDto",
    "BulkUpdateUsersSquadsRequestDto",
    "BulkUpdateUsersSquadsResponseDto",
    "TagStr",
    "UpdateUserFields",
    # Users stats models
    "UserUsageByRange",
    "UserUsageByRangeResponseDto",
    # Config profiles models
    "ConfigProfileDto",
    "CreateConfigProfileRequestDto",
    "CreateConfigProfileResponseDto",
    "DeleteConfigProfileResponseDto",
    "GetAllConfigProfilesResponseDto",
    "GetAllInboundsResponseDto",
    "GetConfigProfileByUuidResponseDto",
    "GetInboundsByProfileUuidResponseDto",
    "InboundDto",
    "NodesProfileDto",
    "ReorderConfigProfileItem",
    "ReorderConfigProfilesRequestDto",
    "ReorderConfigProfilesResponseDto",
    "UpdateConfigProfileRequestDto",
    "UpdateConfigProfileResponseDto",
    "GetAllConfigProfilesResponsePaginated",
    # Infra billing models
    "CreateInfraBillingHistoryRecordRequestDto",
    "CreateInfraBillingHistoryRecordResponseDto", 
    "CreateInfraBillingNodeRequestDto",
    "CreateInfraBillingNodeResponseDto",
    "CreateInfraProviderRequestDto",
    "CreateInfraProviderResponseDto",   
    "DeleteInfraBillingHistoryRecordByUuidResponseDto",  
    "DeleteInfraBillingNodeByUuidResponseDto",  # ПЕРЕИМЕНОВАНА (было DeleteInfraBillingNodeResponseDto)
    "DeleteInfraProviderByUuidResponseDto",  # ПЕРЕИМЕНОВАНА (было DeleteInfraProviderResponseDto)
    "GetInfraBillingHistoryRecordsResponseDto",  # ПЕРЕИМЕНОВАНА (было GetAllInfraBillingHistoryResponseDto)
    "GetInfraBillingNodesResponseDto",  # ПЕРЕИМЕНОВАНА (было GetAllInfraBillingNodesResponseDto)
    "GetInfraProvidersResponseDto",  # ПЕРЕИМЕНОВАНА (было GetAllInfraProvidersResponseDto)
    "GetInfraBillingHistoryByUuidResponseDto",
    "GetInfraBillingNodeByUuidResponseDto",
    "GetInfraProviderByUuidResponseDto",
    "InfraBillingHistoryDto",
    "InfraBillingNodeDto",
    "InfraProviderDto",
    "NodeDto",
    "UpdateInfraBillingNodeRequestDto",
    "UpdateInfraBillingNodeResponseDto",
    "UpdateInfraProviderRequestDto",
    "UpdateInfraProviderResponseDto",
    "DeleteInfraBillingNodeResponseDto",  # LEGACY
    "DeleteInfraProviderResponseDto",  # LEGACY
    "GetAllInfraBillingHistoryResponseDto",  # LEGACY
    "GetAllInfraBillingNodesResponseDto",  # LEGACY
    "GetAllInfraProvidersResponseDto",  # LEGACY
    # Internal squads models
    "AddUsersToInternalSquadRequestDto",
    "AddUsersToInternalSquadResponseDto",
    "CreateInternalSquadRequestDto",
    "CreateInternalSquadResponseDto",
    "DeleteInternalSquadResponseDto",
    "DeleteUsersFromInternalSquadRequestDto",
    "DeleteUsersFromInternalSquadResponseDto",
    "GetAllInternalSquadsResponseDto",
    "GetInternalSquadByUuidResponseDto",
    "InternalSquadDto",
    "ReorderInternalSquadItem",
    "ReorderInternalSquadsRequestDto",
    "ReorderInternalSquadsResponseDto",
    "UpdateInternalSquadRequestDto",
    "UpdateInternalSquadResponseDto",
    "GetInternalSquadAccessibleNodesResponseDto",
    # Nodes usage history models
    "GetNodeUserUsageByRangeResponseDto",
    "GetNodesUsageByRangeResponseDto",
    "GetUserAccessibleNodesResponseDto",
    "NodeInfoDto",
    "NodeUsageDto",
    "UserUsageDto",
    # Subscription request history models
    "GetAllSubscriptionRequestHistoryResponseDto",
    "GetSubscriptionRequestHistoryStatsResponseDto",
    "SubscriptionRequestHistoryRecord",
    "SubscriptionRequestHistoryData",
    "AppStatItem",
    "HourlyRequestStat", 
    "SubscriptionRequestHistoryStatsData",
    # Webhook models
     # USER
    "LastConnectedNodeDto",
    "InternalSquadDto",
    "BaseUserDto",
    "UserDto",
    "UserEventDto",
    "UserTrafficDto",

    # HWID DEVICES
    "HwidUserDeviceDto",
    "UserHwidDeviceEventDto",

    # SERVICE EVENTS
    "LoginAttemptDto",
    "ServiceEventDto",

    # NODE ENTITIES
    "ConfigProfileInboundDto",
    "InfraProviderDto",
    "NodeDto",
    "WebhookNodeConfigProfileDto",
    "NodeEventDto",

    # ERROR EVENTS
    "CustomErrorEventDto",

    # CRM EVENTS
    "CrmEventDto",

    # WEBHOOK PAYLOAD
    "WebhookPayloadDto",
    
    # Passkeys models
    "DeletePasskeyRequestDto",
    "DeletePasskeyResponseDto",
    "GetAllPasskeysResponseDto",
    "GetPasskeyRegistrationOptionsResponseDto",
    "PasskeyDto",
    "UpdatePasskeyRequestDto",
    "UpdatePasskeyResponseDto",
    "VerifyPasskeyRegistrationRequestDto",
    "VerifyPasskeyRegistrationResponseDto",
    
    # External squads models
    "AddUsersToExternalSquadResponseDto",
    "CreateExternalSquadRequestDto",
    "CreateExternalSquadResponseDto",
    "DeleteExternalSquadResponseDto",
    "ExternalSquadDto",
    "ExternalSquadInfoDto",
    "ExternalSquadSubscriptionSettingsDto",
    "ExternalSquadTemplateDto",
    "GetExternalSquadByUuidResponseDto",
    "GetExternalSquadsResponseDto",
    "RemoveUsersFromExternalSquadResponseDto",
    "ReorderExternalSquadItem",
    "ReorderExternalSquadsRequestDto",
    "ReorderExternalSquadsResponseDto",
    "TemplateType",
    "UpdateExternalSquadRequestDto",
    "UpdateExternalSquadResponseDto",
    
    # Snippets models

    "CreateSnippetRequestDto",
    "CreateSnippetResponseDto",
    "DeleteSnippetRequestDto",
    "DeleteSnippetResponseDto",
    "GetSnippetsResponseDto",
    "SnippetItem",
    "SnippetsData",
    "UpdateSnippetRequestDto",
    "UpdateSnippetResponseDto",
    
    # Remnawave settings models
    
    "BrandingSettings",
    "GetRemnawaveSettingsResponseDto",
    "GenericOAuth2Settings",
    "GitHubOAuth2Settings",
    "KeycloakOAuth2Settings",
    "OAuth2Settings",
    "PasskeySettings",
    "PasswordSettings",
    "PocketIdOAuth2Settings",
    "RemnawaveSettingsData",
    "TelegramAuthSettings",
    "UpdateRemnawaveSettingsRequestDto",
    "UpdateRemnawaveSettingsResponseDto",
    "YandexOAuth2Settings",
    
    # Subscription page config models
    "CloneSubscriptionPageConfigRequestDto",
    "CloneSubscriptionPageConfigResponseDto",
    "CreateSubscriptionPageConfigRequestDto",
    "CreateSubscriptionPageConfigResponseDto",
    "DeleteSubscriptionPageConfigResponseDto",
    "GetSubscriptionPageConfigResponseDto",
    "GetSubscriptionPageConfigsResponseDto",
    "GetSubpageConfigByShortUuidRequestBodyDto",
    "GetSubpageConfigByShortUuidResponseDto",
    "SubpageConfigData",
    "ReorderSubscriptionPageConfigItem",
    "ReorderSubscriptionPageConfigsRequestDto",
    "ReorderSubscriptionPageConfigsResponseDto",
    "SubscriptionPageConfigDto",
    "UpdateSubscriptionPageConfigRequestDto",
    "UpdateSubscriptionPageConfigResponseDto",

    # IP Control models
    "DropConnectionsRequestDto",
    "DropByUserUuids",
    "DropByIpAddresses",
    "TargetAllNodes",
    "TargetSpecificNodes",
    "FetchIpsResponseDto",
    "FetchIpsResultResponseDto",
    "DropConnectionsResponseDto",
    "FetchIpsJobData",
    "FetchIpsProgressData",
    "FetchIpsNodeResult",
    "FetchIpsResult",
    "FetchIpsResultData",
    "DropConnectionsResponseData",
]



from typing import Annotated

from rapid_api_client import Query

from remnawave.models import (
    GetAllSubscriptionRequestHistoryResponseDto,
    GetSubscriptionRequestHistoryStatsResponseDto,
)
from remnawave.rapid import BaseController, get


class SubscriptionRequestHistoryController(BaseController):
    @get("/subscription-request-history", response_class=GetAllSubscriptionRequestHistoryResponseDto)
    async def get_all_subscription_request_history(
        self,
        size: Annotated[
            int, Query(default=25, ge=1, description="Page size for pagination")
        ] = 25,
        start: Annotated[
            int, Query(default=0, ge=0, description="Offset for pagination")
        ] = 0,
    ) -> GetAllSubscriptionRequestHistoryResponseDto:
        """Get all subscription request history"""
        ...

    @get(
        "/subscription-request-history/stats",
        response_class=GetSubscriptionRequestHistoryStatsResponseDto,
    )
    async def get_subscription_request_history_stats(
        self,
    ) -> GetSubscriptionRequestHistoryStatsResponseDto:
        """Get subscription request history stats"""
        ...


from enum import StrEnum

class ResponseRuleOperator(StrEnum):
    """Response rule logical operators"""
    AND = "AND"
    OR = "OR"


class ResponseRuleConditionOperator(StrEnum):
    """Response rule condition operators"""
    EQUALS = "EQUALS"
    NOT_EQUALS = "NOT_EQUALS"
    CONTAINS = "CONTAINS"
    NOT_CONTAINS = "NOT_CONTAINS"
    STARTS_WITH = "STARTS_WITH"
    NOT_STARTS_WITH = "NOT_STARTS_WITH"
    ENDS_WITH = "ENDS_WITH"
    NOT_ENDS_WITH = "NOT_ENDS_WITH"
    REGEX = "REGEX"
    NOT_REGEX = "NOT_REGEX"


class ResponseType(StrEnum):
    """Response types for subscription rules"""
    XRAY_JSON = "XRAY_JSON"
    XRAY_BASE64 = "XRAY_BASE64"
    MIHOMO = "MIHOMO"
    STASH = "STASH"
    CLASH = "CLASH"
    SINGBOX = "SINGBOX"
    BROWSER = "BROWSER"
    BLOCK = "BLOCK"
    STATUS_CODE_404 = "STATUS_CODE_404"
    STATUS_CODE_451 = "STATUS_CODE_451"
    SOCKET_DROP = "SOCKET_DROP"


class ResponseRuleVersion(StrEnum):
    """Response rules config version"""
    V1 = "1"


class SubscriptionType(StrEnum):
    """Subscription output types (used e.g. to exclude hosts from specific types)"""
    XRAY_JSON = "XRAY_JSON"
    XRAY_BASE64 = "XRAY_BASE64"
    MIHOMO = "MIHOMO"
    STASH = "STASH"
    CLASH = "CLASH"
    SINGBOX = "SINGBOX"


from enum import StrEnum


class Fingerprint(StrEnum):
    CHROME = "chrome"
    FIREFOX = "firefox"
    SAFARI = "safari"
    IOS = "ios"
    ANDROID = "android"
    EDGE = "edge"
    QQ = "qq"
    RANDOM = "random"
    RANDOMIZED = "randomized"



from datetime import datetime
from typing import Any, List, Optional

from pydantic import AliasChoices, BaseModel, Field

from remnawave.enums import ErrorCode


class ApiErrorResponse(BaseModel):
    """Standard API error response model"""
    timestamp: Optional[datetime] = Field(None, description="Время возникновения ошибки")
    path: Optional[str] = Field(None, description="Путь запроса")
    message: str = Field(..., description="Сообщение об ошибке")
    code: Optional[ErrorCode | str] = Field(
        None,
        validation_alias=AliasChoices("errorCode", "code", "error_code"),
        description="Код ошибки",
    )
    # Support for API v2 error format
    status_code: Optional[int] = Field(None, alias="statusCode")
    errors: Optional[List[Any]] = Field(None, description="Детали ошибок валидации")


class ApiError(Exception):
    """Base API error exception"""
    
    def __init__(self, status_code: int, error: ApiErrorResponse):
        self.status_code = status_code
        self.error = error
        super().__init__(
            f"API Error {error.code}: {error.message} (HTTP {status_code})"
        )

    @property
    def code(self) -> Optional[str]:
        """Get error code"""
        return self.error.code

    @property
    def message(self) -> str:
        """Get error message"""
        return self.error.message

    @property
    def timestamp(self) -> Optional[datetime]:
        """Get error timestamp"""
        return self.error.timestamp

    @property
    def path(self) -> Optional[str]:
        """Get request path"""
        return self.error.path


class BadRequestError(ApiError):
    """Ошибки клиента (400)"""
    pass


class UnauthorizedError(ApiError):
    """Ошибка авторизации (401)"""
    pass


class ForbiddenError(ApiError):
    """Доступ запрещен (403)"""
    pass


class NotFoundError(ApiError):
    """Ресурс не найден (404)"""
    pass


class ConflictError(ApiError):
    """Конфликт (409)"""
    pass


class ValidationError(ApiError):
    """Ошибка валидации данных (422)"""
    pass


class ServerError(ApiError):
    """Серверная ошибка (500+)"""
    pass


# Новые специализированные исключения
class NetworkError(ApiError):
    """Сетевые ошибки"""
    pass


class AuthenticationError(ApiError):
    """Ошибки аутентификации"""
    pass


class BusinessLogicError(ApiError):
    """Ошибки бизнес-логики"""
    pass


class RateLimitError(BadRequestError):
    """Превышен лимит запросов"""
    pass


class MaintenanceError(ServerError):
    """Режим обслуживания"""
    pass


class QuotaExceededError(BusinessLogicError):
    """Превышена квота"""
    pass


class FeatureNotAvailableError(BusinessLogicError):
    """Функция недоступна"""
    pass


from enum import StrEnum


class TemplateType(StrEnum):
    STASH = "STASH"
    SINGBOX = "SINGBOX"
    SINGBOX_LEGACY = "SINGBOX_LEGACY"
    MIHOMO = "MIHOMO"
    XRAY_JSON = "XRAY_JSON"
    CLASH = "CLASH"



from typing import Annotated

from rapid_api_client.annotations import Path, PydanticBody

from remnawave.models import (
    CloneSubscriptionPageConfigRequestDto,
    CloneSubscriptionPageConfigResponseDto,
    CreateSubscriptionPageConfigRequestDto,
    CreateSubscriptionPageConfigResponseDto,
    DeleteSubscriptionPageConfigResponseDto,
    GetSubscriptionPageConfigResponseDto,
    GetSubscriptionPageConfigsResponseDto,
    ReorderSubscriptionPageConfigsRequestDto,
    ReorderSubscriptionPageConfigsResponseDto,
    UpdateSubscriptionPageConfigRequestDto,
    UpdateSubscriptionPageConfigResponseDto,
)
from remnawave.rapid import BaseController, delete, get, patch, post


class SubscriptionPageConfigController(BaseController):
    @get("/subscription-page-configs", response_class=GetSubscriptionPageConfigsResponseDto)
    async def get_all_configs(self) -> GetSubscriptionPageConfigsResponseDto:
        """Get all subscription page configs"""
        ...

    @post("/subscription-page-configs", response_class=CreateSubscriptionPageConfigResponseDto)
    async def create_config(
        self,
        body: Annotated[CreateSubscriptionPageConfigRequestDto, PydanticBody()],
    ) -> CreateSubscriptionPageConfigResponseDto:
        """Create subscription page config"""
        ...

    @patch("/subscription-page-configs", response_class=UpdateSubscriptionPageConfigResponseDto)
    async def update_config(
        self,
        body: Annotated[UpdateSubscriptionPageConfigRequestDto, PydanticBody()],
    ) -> UpdateSubscriptionPageConfigResponseDto:
        """Update subscription page config"""
        ...

    @get("/subscription-page-configs/{uuid}", response_class=GetSubscriptionPageConfigResponseDto)
    async def get_config_by_uuid(
        self,
        uuid: Annotated[str, Path(description="Subscription page config UUID")],
    ) -> GetSubscriptionPageConfigResponseDto:
        """Get subscription page config by uuid"""
        ...

    @delete("/subscription-page-configs/{uuid}", response_class=DeleteSubscriptionPageConfigResponseDto)
    async def delete_config(
        self,
        uuid: Annotated[str, Path(description="Subscription page config UUID")],
    ) -> DeleteSubscriptionPageConfigResponseDto:
        """Delete subscription page config"""
        ...

    @post("/subscription-page-configs/actions/reorder", response_class=ReorderSubscriptionPageConfigsResponseDto)
    async def reorder_configs(
        self,
        body: Annotated[ReorderSubscriptionPageConfigsRequestDto, PydanticBody()],
    ) -> ReorderSubscriptionPageConfigsResponseDto:
        """Reorder subscription page configs"""
        ...

    @post("/subscription-page-configs/actions/clone", response_class=CloneSubscriptionPageConfigResponseDto)
    async def clone_config(
        self,
        body: Annotated[CloneSubscriptionPageConfigRequestDto, PydanticBody()],
    ) -> CloneSubscriptionPageConfigResponseDto:
        """Clone subscription page config"""
        ...


from enum import StrEnum


class ErrorCode(StrEnum):
    INTERNAL_SERVER_ERROR = "A001"
    LOGIN_ERROR = "A002"
    UNAUTHORIZED = "A003"
    FORBIDDEN_ROLE_ERROR = "A004"
    CREATE_API_TOKEN_ERROR = "A005"
    DELETE_API_TOKEN_ERROR = "A006"
    REQUESTED_TOKEN_NOT_FOUND = "A007"
    FIND_ALL_API_TOKENS_ERROR = "A008"
    GET_PUBLIC_KEY_ERROR = "A009"
    ENABLE_NODE_ERROR = "A010"
    NODE_NOT_FOUND = "A011"
    CONFIG_NOT_FOUND = "A012"
    UPDATE_CONFIG_ERROR = "A013"
    GET_CONFIG_ERROR = "A014"
    DELETE_MANY_INBOUNDS_ERROR = "A015"
    CREATE_MANY_INBOUNDS_ERROR = "A016"
    FIND_ALL_INBOUNDS_ERROR = "A017"
    CREATE_USER_ERROR = "A018"
    USER_USERNAME_ALREADY_EXISTS = "A019"
    USER_SHORT_UUID_ALREADY_EXISTS = "A020"
    USER_SUBSCRIPTION_UUID_ALREADY_EXISTS = "A021"
    CREATE_USER_WITH_INBOUNDS_ERROR = "A022"
    CANT_GET_CREATED_USER_WITH_INBOUNDS = "A023"
    GET_ALL_USERS_ERROR = "A024"
    USER_NOT_FOUND = "A025"
    GET_USER_BY_ERROR = "A026"
    REVOKE_USER_SUBSCRIPTION_ERROR = "A027"
    DISABLE_USER_ERROR = "A028"
    USER_ALREADY_DISABLED = "A029"
    USER_ALREADY_ENABLED = "A030"
    ENABLE_USER_ERROR = "A031"
    CREATE_NODE_ERROR = "A032"
    NODE_NAME_ALREADY_EXISTS = "A033"
    NODE_ADDRESS_ALREADY_EXISTS = "A034"
    NODE_ERROR_WITH_MSG = "N001"
    NODE_ERROR_500_WITH_MSG = "N002"
    RESTART_NODE_ERROR = "A035"
    GET_CONFIG_WITH_USERS_ERROR = "A036"
    DELETE_USER_ERROR = "A037"
    UPDATE_NODE_ERROR = "A038"
    UPDATE_USER_ERROR = "A039"
    INCREMENT_USED_TRAFFIC_ERROR = "A040"
    GET_ALL_NODES_ERROR = "A041"
    GET_ONE_NODE_ERROR = "A042"
    DELETE_NODE_ERROR = "A043"
    CREATE_HOST_ERROR = "A044"
    HOST_REMARK_ALREADY_EXISTS = "A045"
    HOST_NOT_FOUND = "A046"
    DELETE_HOST_ERROR = "A047"
    GET_USER_STATS_ERROR = "A048"
    UPDATE_USER_WITH_INBOUNDS_ERROR = "A049"
    GET_ALL_HOSTS_ERROR = "A050"
    REORDER_HOSTS_ERROR = "A051"
    UPDATE_HOST_ERROR = "A052"
    CREATE_CONFIG_ERROR = "A053"
    ENABLED_NODES_NOT_FOUND = "A054"
    GET_NODES_USAGE_BY_RANGE_ERROR = "A055"
    RESET_USER_TRAFFIC_ERROR = "A056"
    REORDER_NODES_ERROR = "A057"
    GET_ALL_INBOUNDS_ERROR = "A058"
    BULK_DELETE_USERS_BY_STATUS_ERROR = "A059"
    UPDATE_INBOUND_ERROR = "A060"
    CONFIG_VALIDATION_ERROR = "A061"
    USERS_NOT_FOUND = "A062"
    GET_USER_BY_UNIQUE_FIELDS_NOT_FOUND = "A063"
    UPDATE_EXCEEDED_TRAFFIC_USERS_ERROR = "A064"
    ADMIN_NOT_FOUND = "A065"
    CREATE_ADMIN_ERROR = "A066"
    GET_AUTH_STATUS_ERROR = "A067"
    FORBIDDEN_ONE = "A068"
    FORBIDDEN_TWO = "E000"
    DISABLE_NODE_ERROR = "A069"
    GET_ONE_HOST_ERROR = "A070"
    SUBSCRIPTION_SETTINGS_NOT_FOUND = "A071"
    GET_SUBSCRIPTION_SETTINGS_ERROR = "A072"
    UPDATE_SUBSCRIPTION_SETTINGS_ERROR = "A073"
    CREATE_INBOUND_ERROR = "A074"
    DELETE_INBOUND_ERROR = "A075"
    GET_INBOUND_ERROR = "A076"
    INBOUND_NOT_FOUND = "A077"
    INBOUND_TAG_ALREADY_EXISTS = "A078"
    CREATE_HOST_BULK_ACTION_ERROR = "A079"
    DELETE_HOST_BULK_ACTION_ERROR = "A080"
    UPDATE_HOST_BULK_ACTION_ERROR = "A081"
    BULK_ACTION_NOT_FOUND = "A082"
    GET_USERS_STATS_ERROR = "A083"
    RESET_USERS_TRAFFIC_BULK_ERROR = "A084"
    UPDATE_USERS_BULK_ERROR = "A085"
    DELETE_USERS_BULK_ERROR = "A086"
    GET_USERS_BULK_ERROR = "A087"
    CREATE_TEMPLATE_ERROR = "A088"
    TEMPLATE_NOT_FOUND = "A089"
    UPDATE_TEMPLATE_ERROR = "A090"
    DELETE_TEMPLATE_ERROR = "A091"
    TEMPLATE_NAME_ALREADY_EXISTS = "A092"
    GET_TEMPLATE_ERROR = "A093"
    GET_ALL_TEMPLATES_ERROR = "A094"
    GENERATE_CONFIG_ERROR = "A095"
    INVALID_TEMPLATE_TYPE = "A096"
    CREATE_EXTERNAL_SQUAD_ERROR = "A097"
    EXTERNAL_SQUAD_NOT_FOUND = "A098"
    UPDATE_EXTERNAL_SQUAD_ERROR = "A099"
    DELETE_EXTERNAL_SQUAD_ERROR = "A100"
    EXTERNAL_SQUAD_NAME_ALREADY_EXISTS = "A101"
    ADD_USERS_TO_EXTERNAL_SQUAD_ERROR = "A102"
    REMOVE_USERS_FROM_EXTERNAL_SQUAD_ERROR = "A103"
    GET_EXTERNAL_SQUAD_ERROR = "A104"
    GET_ALL_EXTERNAL_SQUADS_ERROR = "A105"
    CREATE_INTERNAL_SQUAD_ERROR = "A106"
    INTERNAL_SQUAD_NOT_FOUND = "A107"
    UPDATE_INTERNAL_SQUAD_ERROR = "A108"
    DELETE_INTERNAL_SQUAD_ERROR = "A109"
    INTERNAL_SQUAD_NAME_ALREADY_EXISTS = "A110"
    GET_INTERNAL_SQUAD_ERROR = "A111"
    GET_ALL_INTERNAL_SQUADS_ERROR = "A112"
    CREATE_WEBHOOK_ERROR = "A113"
    WEBHOOK_NOT_FOUND = "A114"
    UPDATE_WEBHOOK_ERROR = "A115"
    DELETE_WEBHOOK_ERROR = "A116"
    WEBHOOK_URL_ALREADY_EXISTS = "A117"
    GET_WEBHOOK_ERROR = "A118"
    GET_ALL_WEBHOOKS_ERROR = "A119"
    WEBHOOK_DELIVERY_ERROR = "A120"
    CREATE_PASSKEY_ERROR = "A121"
    PASSKEY_NOT_FOUND = "A122"
    DELETE_PASSKEY_ERROR = "A123"
    GET_PASSKEY_ERROR = "A124"
    GET_ALL_PASSKEYS_ERROR = "A125"
    PASSKEY_ALREADY_EXISTS = "A126"
    CREATE_SNIPPET_ERROR = "A127"
    SNIPPET_NOT_FOUND = "A128"
    UPDATE_SNIPPET_ERROR = "A129"
    DELETE_SNIPPET_ERROR = "A130"
    SNIPPET_NAME_ALREADY_EXISTS = "A131"
    GET_SNIPPET_ERROR = "A132"
    GET_ALL_SNIPPETS_ERROR = "A133"
    HWID_RESET_ERROR = "A134"
    HWID_NOT_FOUND = "A135"
    GET_HWID_ERROR = "A136"
    GET_ALL_HWIDS_ERROR = "A137"
    DELETE_HWID_ERROR = "A138"
    BANDWIDTH_STATS_ERROR = "A139"
    GET_NODES_USAGE_STATS_ERROR = "A140"
    SUBSCRIPTION_REQUEST_ERROR = "A141"
    SUBSCRIPTION_REQUEST_NOT_FOUND = "A142"
    GET_SUBSCRIPTION_REQUEST_ERROR = "A143"
    GET_ALL_SUBSCRIPTION_REQUESTS_ERROR = "A144"
    APPROVE_SUBSCRIPTION_REQUEST_ERROR = "A145"
    REJECT_SUBSCRIPTION_REQUEST_ERROR = "A146"
    CREATE_SUBSCRIPTION_REQUEST_HISTORY_ERROR = "A147"
    GET_SUBSCRIPTION_REQUEST_HISTORY_ERROR = "A148"
    KEYGEN_ERROR = "A149"
    GENERATE_KEYS_ERROR = "A150"
    INVALID_KEY_TYPE = "A151"
    SYSTEM_STATS_ERROR = "A152"
    SYSTEM_HEALTH_ERROR = "A153"
    NODES_METRICS_ERROR = "A154"
    X25519_KEYGEN_ERROR = "A155"
    HAPP_CRYPTO_ERROR = "A156"
    SRR_MATCHER_ERROR = "A157"
    GET_REMNAWAVE_SETTINGS_ERROR = "A158"
    UPDATE_REMNAWAVE_SETTINGS_ERROR = "A159"
    OAUTH_ERROR = "A160"
    PASSKEY_SETTINGS_ERROR = "A161"
    TELEGRAM_AUTH_ERROR = "A162"
    BRANDING_SETTINGS_ERROR = "A163"
    CONFIG_PROFILE_ERROR = "A164"
    CONFIG_PROFILE_NOT_FOUND = "A165"
    CREATE_CONFIG_PROFILE_ERROR = "A166"
    UPDATE_CONFIG_PROFILE_ERROR = "A167"
    DELETE_CONFIG_PROFILE_ERROR = "A168"
    GET_CONFIG_PROFILE_ERROR = "A169"
    GET_ALL_CONFIG_PROFILES_ERROR = "A170"
    XRAY_CONFIG_ERROR = "A171"
    XRAY_CONFIG_VALIDATION_ERROR = "A172"
    INFRA_BILLING_ERROR = "A173"
    INFRA_BILLING_NOT_FOUND = "A174"
    GET_INFRA_BILLING_ERROR = "A175"
    UPDATE_INFRA_BILLING_ERROR = "A176"
    CALCULATE_BILLING_ERROR = "A177"
    BILLING_PERIOD_ERROR = "A178"
    
    # Добавляем новые коды из failed тестов
    CREATE_SUBSCRIPTION_TEMPLATE_ERROR = "A179"
    SUBSCRIPTION_TEMPLATE_NOT_FOUND = "A180"
    UPDATE_SUBSCRIPTION_TEMPLATE_ERROR = "A181"
    DELETE_SUBSCRIPTION_TEMPLATE_ERROR = "A182"
    GET_SUBSCRIPTION_TEMPLATE_ERROR = "A183"
    
    # Валидационные ошибки
    VALIDATION_ERROR = "V001"
    INVALID_UUID_FORMAT = "V002"
    INVALID_EMAIL_FORMAT = "V003"
    INVALID_DATE_FORMAT = "V004"
    REQUIRED_FIELD_MISSING = "V005"
    FIELD_TOO_LONG = "V006"
    FIELD_TOO_SHORT = "V007"
    INVALID_ENUM_VALUE = "V008"
    INVALID_REGEX_PATTERN = "V009"
    NUMERIC_VALIDATION_ERROR = "V010"
    
    # Сетевые ошибки
    NETWORK_ERROR = "N003"
    TIMEOUT_ERROR = "N004"
    CONNECTION_ERROR = "N005"
    DNS_ERROR = "N006"
    SSL_ERROR = "N007"
    
    # Ошибки аутентификации и авторизации
    INVALID_TOKEN = "AUTH001"
    TOKEN_EXPIRED = "AUTH002"
    INVALID_CREDENTIALS = "AUTH003"
    TWO_FACTOR_REQUIRED = "AUTH004"
    ACCOUNT_LOCKED = "AUTH005"
    PASSWORD_COMPLEXITY_ERROR = "AUTH006"
    
    # Ошибки бизнес-логики
    TRAFFIC_LIMIT_EXCEEDED = "BL001"
    USER_LIMIT_EXCEEDED = "BL002"
    SUBSCRIPTION_EXPIRED = "BL003"
    FEATURE_NOT_AVAILABLE = "BL004"
    QUOTA_EXCEEDED = "BL005"
    RESOURCE_LOCKED = "BL006"
    
    # Общие коды
    UNKNOWN = "UNKNOWN"
    NOT_IMPLEMENTED = "NOT_IMPLEMENTED"
    MAINTENANCE_MODE = "MAINTENANCE"
    RATE_LIMIT_EXCEEDED = "RATE_LIMIT"


from typing import Annotated, List
from uuid import UUID

from rapid_api_client.annotations import PydanticBody

from remnawave.enums import UserStatus
from remnawave.models import (
    BulkAllExtendExpirationDateRequestDto,
    BulkAllExtendExpirationDateResponseDto,
    BulkAllResetTrafficUsersResponseDto,
    BulkAllUpdateUsersRequestDto,
    BulkAllUpdateUsersResponseDto,
    BulkDeleteUsersByStatusRequestDto,
    BulkDeleteUsersByStatusResponseDto,
    BulkDeleteUsersRequestDto,
    BulkDeleteUsersResponseDto,
    BulkExtendExpirationDateRequestDto,
    BulkExtendExpirationDateResponseDto,
    BulkResetTrafficUsersRequestDto,
    BulkResetTrafficUsersResponseDto,
    BulkRevokeUsersSubscriptionRequestDto,
    BulkRevokeUsersSubscriptionResponseDto,
    BulkUpdateUsersRequestDto,
    BulkUpdateUsersResponseDto,
    BulkUpdateUsersSquadsRequestDto,
    BulkUpdateUsersSquadsResponseDto,
)
from remnawave.rapid import BaseController, post


class UsersBulkActionsController(BaseController):
    @post(
        "/users/bulk/delete-by-status",
        response_class=BulkDeleteUsersByStatusResponseDto,
    )
    async def bulk_delete_users_by_status(
        self, 
        body: Annotated[BulkDeleteUsersByStatusRequestDto, PydanticBody()]
    ) -> BulkDeleteUsersByStatusResponseDto:
        """Bulk Delete Users By Status"""
        ...

    @post("/users/bulk/delete", response_class=BulkDeleteUsersResponseDto)
    async def bulk_delete_users(
        self,
        body: Annotated[BulkDeleteUsersRequestDto, PydanticBody()],
    ) -> BulkDeleteUsersResponseDto:
        """Bulk Delete Users By UUIDs"""
        ...

    @post(
        "/users/bulk/revoke-subscription",
        response_class=BulkRevokeUsersSubscriptionResponseDto,
    )
    async def bulk_revoke_users_subscription(
        self,
        body: Annotated[BulkRevokeUsersSubscriptionRequestDto, PydanticBody()],
    ) -> BulkRevokeUsersSubscriptionResponseDto:
        """Bulk Revoke Users Subscription"""
        ...

    @post("/users/bulk/reset-traffic", response_class=BulkResetTrafficUsersResponseDto)
    async def bulk_reset_user_traffic(
        self,
        body: Annotated[BulkResetTrafficUsersRequestDto, PydanticBody()],
    ) -> BulkResetTrafficUsersResponseDto:
        """Bulk Reset User Traffic"""
        ...

    @post("/users/bulk/update", response_class=BulkUpdateUsersResponseDto)
    async def bulk_update_users(
        self,
        body: Annotated[BulkUpdateUsersRequestDto, PydanticBody()],
    ) -> BulkUpdateUsersResponseDto:
        """Bulk Update Users"""
        ...

    @post("/users/bulk/update-squads", response_class=BulkUpdateUsersSquadsResponseDto)
    async def bulk_update_users_internal_squads(
        self,
        body: Annotated[BulkUpdateUsersSquadsRequestDto, PydanticBody()],
    ) -> BulkUpdateUsersSquadsResponseDto:
        """Bulk Update Users Internal Squads"""
        ...

    @post(
        "/users/bulk/extend-expiration-date",
        response_class=BulkExtendExpirationDateResponseDto,
    )
    async def bulk_extend_expiration_date(
        self,
        body: Annotated[BulkExtendExpirationDateRequestDto, PydanticBody()],
    ) -> BulkExtendExpirationDateResponseDto:
        """Bulk Extend Users Expiration Date"""
        ...

    @post("/users/bulk/all/update", response_class=BulkAllUpdateUsersResponseDto)
    async def bulk_update_all_users(
        self,
        body: Annotated[BulkAllUpdateUsersRequestDto, PydanticBody()],
    ) -> BulkAllUpdateUsersResponseDto:
        """Bulk Update All Users"""
        ...

    @post(
        "/users/bulk/all/reset-traffic",
        response_class=BulkAllResetTrafficUsersResponseDto,
    )
    async def bulk_all_reset_user_traffic(
        self,
    ) -> BulkAllResetTrafficUsersResponseDto:
        """Bulk Reset All Users Traffic"""
        ...

    @post(
        "/users/bulk/all/extend-expiration-date",
        response_class=BulkAllExtendExpirationDateResponseDto,
    )
    async def bulk_all_extend_expiration_date(
        self,
        body: Annotated[BulkAllExtendExpirationDateRequestDto, PydanticBody()],
    ) -> BulkAllExtendExpirationDateResponseDto:
        """Bulk Extend All Users Expiration Date"""
        ...


from typing import Annotated

from rapid_api_client import Path
from rapid_api_client.annotations import PydanticBody

from remnawave.models import (
    DropConnectionsRequestDto,
    DropConnectionsResponseDto,
    FetchIpsResponseDto,
    FetchIpsResultResponseDto,
)
from remnawave.rapid import BaseController, get, post


class IpControlController(BaseController):
    @post("/ip-control/fetch-ips/{uuid}", response_class=FetchIpsResponseDto)
    async def fetch_user_ips(
        self,
        uuid: Annotated[str, Path(description="UUID of the user")],
    ) -> FetchIpsResponseDto:
        """Request IP List for User.

        Starts a background job that queries all connected nodes for the IPs
        used by the given user.  The returned ``job_id`` must be passed to
        :meth:`get_fetch_ips_result` to retrieve the actual list once the job
        is complete.
        """
        ...

    @get("/ip-control/fetch-ips/result/{jobId}", response_class=FetchIpsResultResponseDto)
    async def get_fetch_ips_result(
        self,
        jobId: Annotated[str, Path(description="Job ID returned by fetch_user_ips")],
    ) -> FetchIpsResultResponseDto:
        """Get IP List Result by Job ID.

        Poll this endpoint after calling :meth:`fetch_user_ips`.  When
        ``is_completed`` is ``True`` the ``result`` field contains per-node
        IP lists.  When ``is_failed`` is ``True`` the job encountered an
        error.
        """
        ...

    @post("/ip-control/drop-connections", response_class=DropConnectionsResponseDto)
    async def drop_connections(
        self,
        body: Annotated[DropConnectionsRequestDto, PydanticBody()],
    ) -> DropConnectionsResponseDto:
        """Drop active connections.

        Sends a drop-connections event to the target nodes.  You can specify
        the connections to drop either by user UUIDs or by IP addresses, and
        you can target all connected nodes or a specific subset.
        """
        ...



from datetime import datetime
from typing import Annotated, List, Optional
from uuid import UUID

from pydantic import BaseModel, Field

from remnawave.enums import UserStatus, TrafficLimitStrategy


# Type alias for tag validation
TagStr = Annotated[
    str,
    Field(min_length=1, max_length=16, pattern=r"^[A-Z0-9_]+$")
]


# Request DTOs
class BulkDeleteUsersByStatusRequestDto(BaseModel):
    """Request to delete users by status"""
    status: UserStatus = Field(default=UserStatus.ACTIVE)


class BulkDeleteUsersRequestDto(BaseModel):
    """Request to delete users by UUIDs"""
    uuids: List[UUID] = Field(min_length=1, max_length=500)


class BulkRevokeUsersSubscriptionRequestDto(BaseModel):
    """Request to revoke users subscription"""
    uuids: List[UUID] = Field(min_length=1, max_length=500)


class BulkResetTrafficUsersRequestDto(BaseModel):
    """Request to reset traffic for users"""
    uuids: List[UUID] = Field(min_length=1, max_length=500)


class UpdateUserFields(BaseModel):
    """Fields to update for users"""
    status: Optional[UserStatus] = None
    traffic_limit_bytes: Optional[int] = Field(
        None, 
        serialization_alias="trafficLimitBytes", 
        ge=0,
        description="Traffic limit in bytes. 0 - unlimited"
    )
    traffic_limit_strategy: Optional[TrafficLimitStrategy] = Field(
        None, 
        serialization_alias="trafficLimitStrategy",
        description="Traffic limit reset strategy"
    )
    expire_at: Optional[datetime] = Field(
        None, 
        serialization_alias="expireAt",
        description="Expiration date: 2025-01-17T15:38:45.065Z"
    )
    description: Optional[str] = None
    telegram_id: Optional[int] = Field(None, serialization_alias="telegramId")
    email: Optional[str] = None
    tag: Optional[TagStr] = Field(
        None,
        description="Tag for user. Must be uppercase, alphanumeric, and can include underscores. Max length 16 characters."
    )
    hwid_device_limit: Optional[int] = Field(
        None, 
        serialization_alias="hwidDeviceLimit", 
        ge=0
    )
    external_squad_uuid: Optional[UUID] = Field(
        None,
        serialization_alias="externalSquadUuid",
        description="Optional. External squad UUID."
    )


class BulkUpdateUsersRequestDto(BaseModel):
    """Request to bulk update users"""
    uuids: List[UUID] = Field(min_length=1, max_length=500)
    fields: UpdateUserFields


class BulkUpdateUsersSquadsRequestDto(BaseModel):
    """Request to update users internal squads"""
    uuids: List[UUID] = Field(min_length=1, max_length=500)
    active_internal_squads: List[UUID] = Field(serialization_alias="activeInternalSquads")


class BulkExtendExpirationDateRequestDto(BaseModel):
    """Request to extend expiration date for selected users"""
    uuids: List[UUID] = Field(min_length=1, max_length=500)
    extend_days: int = Field(
        serialization_alias="extendDays",
        ge=1,
        le=9999
    )


class BulkAllUpdateUsersRequestDto(BaseModel):
    """Request to update all users"""
    status: Optional[UserStatus] = Field(default=UserStatus.ACTIVE)
    traffic_limit_bytes: Optional[int] = Field(
        None,
        serialization_alias="trafficLimitBytes",
        ge=0,
        description="Traffic limit in bytes. 0 - unlimited"
    )
    traffic_limit_strategy: Optional[TrafficLimitStrategy] = Field(
        None,
        serialization_alias="trafficLimitStrategy",
        description="Traffic limit reset strategy"
    )
    expire_at: Optional[datetime] = Field(
        None,
        serialization_alias="expireAt",
        description="Expiration date: 2025-01-17T15:38:45.065Z"
    )
    description: Optional[str] = None
    telegram_id: Optional[int] = Field(None, serialization_alias="telegramId")
    email: Optional[str] = None
    tag: Optional[TagStr] = Field(
        None,
        description="Tag for user. Must be uppercase, alphanumeric, and can include underscores. Max length 16 characters."
    )
    hwid_device_limit: Optional[int] = Field(
        None, 
        serialization_alias="hwidDeviceLimit", 
        ge=0
    )


class BulkAllExtendExpirationDateRequestDto(BaseModel):
    """Request to extend expiration date for all users"""
    extend_days: int = Field(
        serialization_alias="extendDays",
        ge=1
    )


# Base Response DTOs (без обертки response)
class BulkResponseData(BaseModel):
    """Common bulk response with affected rows"""
    affected_rows: int = Field(alias="affectedRows")


class BulkEventResponseData(BaseModel):
    """Common bulk response with event sent flag"""
    event_sent: bool = Field(alias="eventSent")


# Response DTOs - наследуются от базовых
class BulkDeleteUsersByStatusResponseDto(BulkResponseData):
    """Response for bulk delete by status"""
    pass


class BulkDeleteUsersResponseDto(BulkResponseData):
    """Response for bulk delete users"""
    pass


class BulkRevokeUsersSubscriptionResponseDto(BulkResponseData):
    """Response for bulk revoke subscription"""
    pass


class BulkResetTrafficUsersResponseDto(BulkResponseData):
    """Response for bulk reset traffic"""
    pass


class BulkUpdateUsersResponseDto(BulkResponseData):
    """Response for bulk update users"""
    pass


class BulkUpdateUsersSquadsResponseDto(BulkResponseData):
    """Response for bulk update squads"""
    pass


class BulkExtendExpirationDateResponseDto(BulkResponseData):
    """Response for bulk extend expiration date"""
    pass


class BulkAllUpdateUsersResponseDto(BulkEventResponseData):
    """Response for bulk update all users"""
    pass


class BulkAllResetTrafficUsersResponseDto(BulkEventResponseData):
    """Response for bulk reset all users traffic"""
    pass


class BulkAllExtendExpirationDateResponseDto(BulkEventResponseData):
    """Response for bulk extend all users expiration date"""
    pass


# Legacy compatibility
BulkResponseDto = BulkResponseData
BulkUpdateUsersInternalSquadsRequestDto = BulkUpdateUsersSquadsRequestDto